import typing, abc
from System.ComponentModel import INotifyPropertyChanged
from System.Collections.ObjectModel import ObservableCollection_1, ReadOnlyObservableCollection_1
from SIMULTAN.Projects import ProjectData
from System.Collections.Generic import Dictionary_2, List_1, IEnumerable_1
from SIMULTAN.Utils.Collections import ElectivelyObservableCollection_1, IReadonlyObservableDictionary_2
from System import ValueTuple_2, ValueTuple_3, MulticastDelegate, IAsyncResult, AsyncCallback, Exception, IEquatable_1
from System.IO import FileInfo, DirectoryInfo, FileSystemInfo
from SIMULTAN.Data.Users import SimUser, SimUserRole, SimUsersManager
from SIMULTAN.Data.Components import SimComponent, SimComponentVisibility
from System.Collections.Specialized import NotifyCollectionChangedEventArgs
from System.Reflection import MethodInfo, MethodBase
from SIMULTAN.Data.Taxonomy import SimTaxonomyEntryReferenceCollection
from System.Collections import IDictionary

class Asset(INotifyPropertyChanged, abc.ABC):
    @property
    def ContainedObjectId(self) -> str: ...
    @ContainedObjectId.setter
    def ContainedObjectId(self, value: str) -> str: ...
    @property
    def ReferencingComponentIds(self) -> ObservableCollection_1[int]: ...
    @property
    def Resource(self) -> ResourceEntry: ...
    @property
    def ResourceKey(self) -> int: ...
    @ResourceKey.setter
    def ResourceKey(self, value: int) -> int: ...
    def AddReferencing(self, _id: int) -> int: ...
    def GetAssetContent(self, _look_in_cache_first: bool) -> typing.Any: ...
    def IsBeingReferencedBy(self, _id: int) -> bool: ...
    def OpenAssetContent(self) -> typing.Any: ...
    def RemoveReferencing(self, _id: int) -> int: ...
    def ToString(self) -> str: ...


class AssetManager:
    def __init__(self, projectData: ProjectData) -> None: ...
    PATH_NOT_FOUND : str
    @property
    def Assets(self) -> Dictionary_2[int, ElectivelyObservableCollection_1[Asset]]: ...
    @Assets.setter
    def Assets(self, value: Dictionary_2[int, ElectivelyObservableCollection_1[Asset]]) -> Dictionary_2[int, ElectivelyObservableCollection_1[Asset]]: ...
    @property
    def MissingLinkedResources(self) -> List_1[LinkedResourceFileEntry]: ...
    @property
    def PathsToResourceFiles(self) -> PathsToLinkedResourcesCollection: ...
    @PathsToResourceFiles.setter
    def PathsToResourceFiles(self, value: PathsToLinkedResourcesCollection) -> PathsToLinkedResourcesCollection: ...
    @property
    def ProjectData(self) -> ProjectData: ...
    @property
    def Resources(self) -> ReadOnlyObservableCollection_1[ResourceEntry]: ...
    @property
    def WorkingDirectory(self) -> str: ...
    @WorkingDirectory.setter
    def WorkingDirectory(self, value: str) -> str: ...
    def CanCopyResourceEntry(self, _original: ResourceEntry, _target: ResourceDirectoryEntry, copyNameFormat: str) -> ValueTuple_2[bool, str]: ...
    def CanLinkAsResource(self, _file: FileInfo) -> ResourceLocationError: ...
    def ContainedToLinked(self, _original: ContainedResourceFileEntry, _target: DirectoryInfo, copyNameFormat: str) -> LinkedResourceFileEntry: ...
    def CopyResourceAsContainedFileEntry(self, sourceFile: FileInfo, targetDir: DirectoryInfo, copyNameFormat: str) -> ValueTuple_2[int, FileInfo]: ...
    def CopyResourceEntry(self, _original: ResourceEntry, _target: ResourceDirectoryEntry, copyNameFormat: str, newOwner: SimUser = ...) -> ValueTuple_2[int, FileSystemInfo]: ...
    def CreateDocumentAsset(self, Component: SimComponent, file: ResourceFileEntry, _id_contained: str) -> DocumentAsset: ...
    def CreateGeometricAsset(self, componentId: int, resourceKey: int, geometryId: str) -> GeometricAsset: ...
    def CreateResourceDirIn(self, _name: str, _parent: DirectoryInfo, collisionNameFormat: str) -> ValueTuple_2[int, DirectoryInfo]: ...
    def DeleteResourceEntryAny(self, _to_delete: ResourceEntry) -> bool: ...
    def DeleteResourceEntryPossible(self, _to_delete: ResourceEntry) -> ValueTuple_2[bool, bool]: ...
    def DeleteResourceFileEntry(self, _current_full_path_to_resource: str, _name_of_resource: str) -> int: ...
    def GetAllLinkedFiles(self) -> List_1[FileInfo]: ...
    def GetAllResourceFiles(self, _extensions: IEnumerable_1[str], _contained: bool) -> List_1[FileInfo]: ...
    def GetFileExtension(self, _key: int, _with_dot: bool = ...) -> str: ...
    def GetFirstResourceWithTag(self, taxonomyKey: str, taxonomyEntryKey: str) -> ResourceFileEntry: ...
    def GetKey(self, _file_name: str) -> int: ...
    def GetPath(self, _key: int) -> str: ...
    def GetResourceKey(self, _file: FileInfo) -> int: ...
    def IsValidResourcePath(self, _fi: FileInfo, _is_contained: bool) -> bool: ...
    def LinkedToContained(self, _original: LinkedResourceFileEntry, nameFormat: str) -> ContainedResourceFileEntry: ...
    def OnChildResourceCollectionChanged(self, args: NotifyCollectionChangedEventArgs) -> None: ...
    def OnResourceRenamed(self, oldName: str, newName: str) -> None: ...
    def OnUpToDate(self) -> None: ...
    def ReLinkLinkedFileEntry(self, _resource: LinkedResourceFileEntry, _new_file: FileInfo, _allow_duplicates: bool = ...) -> bool: ...
    def RemoveAsset(self, _a: Asset) -> None: ...
    def Reset(self) -> None: ...
    def ResetLinks(self) -> None: ...
    def ResourceEntryDeltedExternally(self, _to_delete: ResourceEntry) -> bool: ...
    def ResourceFileEntryExists(self, file: FileInfo) -> bool: ...
    def RestoreDefaultTaxonomyReferences(self) -> None: ...
    def SynchronizeResources(self, _resource_files: IEnumerable_1[FileInfo], _resource_dirs: IEnumerable_1[DirectoryInfo], _create_missing: bool, _delete_superfluous: bool) -> ValueTuple_3[List_1[ResourceEntry], List_1[FileInfo], List_1[DirectoryInfo]]: ...
    def UnLinkLinkedFileEntry(self, _resource: LinkedResourceFileEntry) -> None: ...
    # Skipped AddResourceEntry due to it being static, abstract and generic.

    AddResourceEntry : AddResourceEntry_MethodGroup
    class AddResourceEntry_MethodGroup:
        @typing.overload
        def __call__(self, file: FileInfo) -> int:...
        @typing.overload
        def __call__(self, directory: ResourceDirectoryEntry) -> None:...
        @typing.overload
        def __call__(self, file: ResourceFileEntry, parent: ResourceDirectoryEntry) -> None:...

    # Skipped GetResource due to it being static, abstract and generic.

    GetResource : GetResource_MethodGroup
    class GetResource_MethodGroup:
        @typing.overload
        def __call__(self, _key: int) -> ResourceEntry:...
        @typing.overload
        def __call__(self, _dir: DirectoryInfo) -> ResourceDirectoryEntry:...
        @typing.overload
        def __call__(self, _file: FileInfo) -> ResourceFileEntry:...

    # Skipped LinkResourceAsLinkedFileEntry due to it being static, abstract and generic.

    LinkResourceAsLinkedFileEntry : LinkResourceAsLinkedFileEntry_MethodGroup
    class LinkResourceAsLinkedFileEntry_MethodGroup:
        @typing.overload
        def __call__(self, sourceFile: FileInfo, directory: ResourceDirectoryEntry) -> ResourceFileEntry:...
        @typing.overload
        def __call__(self, _source_file: FileInfo, _parent: DirectoryInfo, _allow_duplicates: bool = ...) -> ValueTuple_2[int, bool]:...


    class ChildResourceCollectionChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, args: NotifyCollectionChangedEventArgs, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, args: NotifyCollectionChangedEventArgs) -> None: ...


    class ResourceDeletedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, entry: ResourceEntry, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, entry: ResourceEntry) -> None: ...


    class ResourceRenamedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, oldName: str, newName: str, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, oldName: str, newName: str) -> None: ...


    class UpToDateEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...



class AttemptedToAddInadmissibleEventHandler_GenericClasses(abc.ABCMeta):
    Generic_AttemptedToAddInadmissibleEventHandler_GenericClasses_AttemptedToAddInadmissibleEventHandler_1_T = typing.TypeVar('Generic_AttemptedToAddInadmissibleEventHandler_GenericClasses_AttemptedToAddInadmissibleEventHandler_1_T')
    def __getitem__(self, types : typing.Type[Generic_AttemptedToAddInadmissibleEventHandler_GenericClasses_AttemptedToAddInadmissibleEventHandler_1_T]) -> typing.Type[AttemptedToAddInadmissibleEventHandler_1[Generic_AttemptedToAddInadmissibleEventHandler_GenericClasses_AttemptedToAddInadmissibleEventHandler_1_T]]: ...

AttemptedToAddInadmissibleEventHandler : AttemptedToAddInadmissibleEventHandler_GenericClasses

AttemptedToAddInadmissibleEventHandler_1_T = typing.TypeVar('AttemptedToAddInadmissibleEventHandler_1_T')
class AttemptedToAddInadmissibleEventHandler_1(typing.Generic[AttemptedToAddInadmissibleEventHandler_1_T], MulticastDelegate):
    def __init__(self, object: typing.Any, method: int) -> None: ...
    @property
    def Method(self) -> MethodInfo: ...
    @property
    def Target(self) -> typing.Any: ...
    def BeginInvoke(self, sender: typing.Any, index: int, item: AttemptedToAddInadmissibleEventHandler_1_T, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
    def EndInvoke(self, result: IAsyncResult) -> None: ...
    def Invoke(self, sender: typing.Any, index: int, item: AttemptedToAddInadmissibleEventHandler_1_T) -> None: ...


class ContainedResourceFileEntry(ResourceFileEntry):
    @property
    def CanBeMoved(self) -> bool: ...
    @property
    def CanBeRenamed(self) -> bool: ...
    @property
    def CurrentFullPath(self) -> str: ...
    @CurrentFullPath.setter
    def CurrentFullPath(self, value: str) -> str: ...
    @property
    def CurrentRelativePath(self) -> str: ...
    @CurrentRelativePath.setter
    def CurrentRelativePath(self, value: str) -> str: ...
    @property
    def Exists(self) -> bool: ...
    @property
    def Extension(self) -> str: ...
    @property
    def FileIsNotReallyContained(self) -> bool: ...
    @FileIsNotReallyContained.setter
    def FileIsNotReallyContained(self, value: bool) -> bool: ...
    @property
    def IsMissing(self) -> bool: ...
    @IsMissing.setter
    def IsMissing(self, value: bool) -> bool: ...
    @property
    def Key(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def NameWithoutExtension(self) -> str: ...
    @property
    def Parent(self) -> ResourceEntry: ...
    @Parent.setter
    def Parent(self, value: ResourceEntry) -> ResourceEntry: ...
    @property
    def Tags(self) -> SimTaxonomyEntryReferenceCollection: ...
    @property
    def UserWithWritingAccess(self) -> SimUserRole: ...
    @UserWithWritingAccess.setter
    def UserWithWritingAccess(self, value: SimUserRole) -> SimUserRole: ...
    @property
    def Visibility(self) -> SimComponentVisibility: ...
    @Visibility.setter
    def Visibility(self, value: SimComponentVisibility) -> SimComponentVisibility: ...
    def CanChangeLocation(self, _new_location: DirectoryInfo, nameCollisionFormat: str) -> ValueTuple_2[bool, str]: ...
    def CanChangePath(self, _new_data: FileSystemInfo, nameCollisionFormat: str) -> ValueTuple_2[bool, str]: ...
    def CanReplacePath(self, _replacement_path: str) -> bool: ...
    def ChangeLocation(self, _new_location: DirectoryInfo, nameCollisionFormat: str, _check_admissibility: bool) -> None: ...
    def ChangeName(self, _new_name: str, nameCollisionFormat: str) -> None: ...
    def ChangePath(self, _new_data: FileSystemInfo, nameCollisionFormat: str, _check_admissibility: bool) -> None: ...


class DefaultMachineHashGenerator(IMachineHashGenerator):
    def __init__(self) -> None: ...
    def GetMachineHash(self) -> str: ...
    def GetMachineName(self) -> str: ...
    def GetUserDomainName(self) -> str: ...
    def GetUserName(self) -> str: ...


class DocumentAsset(Asset):
    @property
    def ContainedObjectId(self) -> str: ...
    @ContainedObjectId.setter
    def ContainedObjectId(self, value: str) -> str: ...
    @property
    def ReferencingComponentIds(self) -> ObservableCollection_1[int]: ...
    @property
    def Resource(self) -> ResourceEntry: ...
    @property
    def ResourceKey(self) -> int: ...
    @ResourceKey.setter
    def ResourceKey(self, value: int) -> int: ...


class GeometricAsset(Asset):
    @property
    def ContainedObjectId(self) -> str: ...
    @ContainedObjectId.setter
    def ContainedObjectId(self, value: str) -> str: ...
    @property
    def ReferencingComponentIds(self) -> ObservableCollection_1[int]: ...
    @property
    def Resource(self) -> ResourceEntry: ...
    @property
    def ResourceKey(self) -> int: ...
    @ResourceKey.setter
    def ResourceKey(self, value: int) -> int: ...
    def OpenAssetContent(self) -> typing.Any: ...


class IMachineHashGenerator(typing.Protocol):
    @abc.abstractmethod
    def GetMachineHash(self) -> str: ...


class InvalidPathException(Exception):
    def __init__(self) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class LinkedResourceFileEntry(ResourceFileEntry):
    @property
    def CanBeMoved(self) -> bool: ...
    @property
    def CanBeRenamed(self) -> bool: ...
    @property
    def CurrentFullPath(self) -> str: ...
    @CurrentFullPath.setter
    def CurrentFullPath(self, value: str) -> str: ...
    @property
    def CurrentRelativePath(self) -> str: ...
    @CurrentRelativePath.setter
    def CurrentRelativePath(self, value: str) -> str: ...
    @property
    def Exists(self) -> bool: ...
    @property
    def Extension(self) -> str: ...
    @property
    def Key(self) -> int: ...
    @property
    def MoreThanOneValidPathDetected(self) -> bool: ...
    @MoreThanOneValidPathDetected.setter
    def MoreThanOneValidPathDetected(self, value: bool) -> bool: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def NameWithoutExtension(self) -> str: ...
    @property
    def Parent(self) -> ResourceEntry: ...
    @Parent.setter
    def Parent(self, value: ResourceEntry) -> ResourceEntry: ...
    @property
    def Tags(self) -> SimTaxonomyEntryReferenceCollection: ...
    @property
    def UserWithWritingAccess(self) -> SimUserRole: ...
    @UserWithWritingAccess.setter
    def UserWithWritingAccess(self, value: SimUserRole) -> SimUserRole: ...
    @property
    def Visibility(self) -> SimComponentVisibility: ...
    @Visibility.setter
    def Visibility(self, value: SimComponentVisibility) -> SimComponentVisibility: ...
    def CanChangeLocation(self, _new_location: DirectoryInfo, nameCollisionFormat: str) -> ValueTuple_2[bool, str]: ...
    def CanReplacePath(self, _replacement_path: str) -> bool: ...
    def ChangeLocation(self, _new_location: DirectoryInfo, nameCollisionFormat: str, _check_admissibility: bool) -> None: ...


class MultiLink(IEquatable_1[MultiLink]):
    def __init__(self, _full_path: str) -> None: ...
    @property
    def IsEmpty(self) -> bool: ...
    @IsEmpty.setter
    def IsEmpty(self, value: bool) -> bool: ...
    @property
    def Representations(self) -> IReadonlyObservableDictionary_2[str, str]: ...
    def AddRepresentation(self, _full_path: str) -> bool: ...
    def GetHashCode(self) -> int: ...
    def GetLink(self) -> str: ...
    def HasPath(self, _full_path: str) -> bool: ...
    def IsEquivalent(self, _other: MultiLink) -> bool: ...
    def __eq__(self, _mL1: MultiLink, _mL2: MultiLink) -> bool: ...
    def __ne__(self, _mL1: MultiLink, _mL2: MultiLink) -> bool: ...
    def RemoveRepresentation(self, _full_path: str) -> bool: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, _ml: MultiLink) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...



class MultiLinkManager:
    def __init__(self) -> None: ...
    @property
    def Links(self) -> ObservableCollection_1[MultiLink]: ...
    @Links.setter
    def Links(self, value: ObservableCollection_1[MultiLink]) -> ObservableCollection_1[MultiLink]: ...
    @property
    def SecondaryDataManager(self) -> AssetManager: ...
    @SecondaryDataManager.setter
    def SecondaryDataManager(self, value: AssetManager) -> AssetManager: ...
    @property
    def UserEncryptionUtiliy(self) -> SimUsersManager: ...
    @UserEncryptionUtiliy.setter
    def UserEncryptionUtiliy(self, value: SimUsersManager) -> SimUsersManager: ...
    def Clear(self) -> None: ...
    def GetLinksFromAssetManager(self) -> None: ...


class PathsToLinkedResourcesCollection(ElectivelyObservableCollection_1[str]):
    def __init__(self, forbiddenFolder: str) -> None: ...
    @property
    def Count(self) -> int: ...
    @property
    def ForbiddenFolder(self) -> str: ...
    @ForbiddenFolder.setter
    def ForbiddenFolder(self, value: str) -> str: ...
    @property
    def Item(self) -> str: ...
    @Item.setter
    def Item(self, value: str) -> str: ...
    @property
    def SuppressNotification(self) -> bool: ...
    @SuppressNotification.setter
    def SuppressNotification(self, value: bool) -> bool: ...


class ResourceDirectoryEntry(ResourceEntry):
    @property
    def CanBeMoved(self) -> bool: ...
    @property
    def CanBeRenamed(self) -> bool: ...
    @property
    def Children(self) -> ElectivelyObservableCollection_1[ResourceEntry]: ...
    @property
    def CurrentFullPath(self) -> str: ...
    @CurrentFullPath.setter
    def CurrentFullPath(self, value: str) -> str: ...
    @property
    def CurrentRelativePath(self) -> str: ...
    @CurrentRelativePath.setter
    def CurrentRelativePath(self, value: str) -> str: ...
    @property
    def Exists(self) -> bool: ...
    @property
    def Key(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Parent(self) -> ResourceEntry: ...
    @Parent.setter
    def Parent(self, value: ResourceEntry) -> ResourceEntry: ...
    @property
    def Tags(self) -> SimTaxonomyEntryReferenceCollection: ...
    @property
    def UserWithWritingAccess(self) -> SimUserRole: ...
    @UserWithWritingAccess.setter
    def UserWithWritingAccess(self, value: SimUserRole) -> SimUserRole: ...
    @property
    def Visibility(self) -> SimComponentVisibility: ...
    @Visibility.setter
    def Visibility(self, value: SimComponentVisibility) -> SimComponentVisibility: ...
    def CanChangeLocation(self, _new_location: DirectoryInfo, nameCollisionFormat: str) -> ValueTuple_2[bool, str]: ...
    def CanChangePath(self, _new_data: FileSystemInfo, nameCollisionFormat: str) -> ValueTuple_2[bool, str]: ...
    def CanReplacePath(self, _replacement_path: str) -> bool: ...
    def ChangeLocation(self, _new_location: DirectoryInfo, nameCollisionFormat: str, _check_admissibility: bool) -> None: ...
    def ChangeName(self, _new_name: str, nameCollisionFormat: str) -> None: ...
    def ChangePath(self, _new_data: FileSystemInfo, nameCollisionFormat: str, _check_admissibility: bool) -> None: ...
    def DeepestContainingEntry(self, _dir_of_new_entry: DirectoryInfo) -> ResourceDirectoryEntry: ...
    def GetContainedKeys(self) -> List_1[int]: ...
    def GetFlatContainedContent(self) -> Dictionary_2[int, FileSystemInfo]: ...
    def GetFlatLinks(self) -> List_1[FileInfo]: ...


class ResourceEntry(IEquatable_1[ResourceEntry], INotifyPropertyChanged, abc.ABC):
    @property
    def CanBeMoved(self) -> bool: ...
    @property
    def CanBeRenamed(self) -> bool: ...
    @property
    def CurrentFullPath(self) -> str: ...
    @CurrentFullPath.setter
    def CurrentFullPath(self, value: str) -> str: ...
    @property
    def CurrentRelativePath(self) -> str: ...
    @CurrentRelativePath.setter
    def CurrentRelativePath(self, value: str) -> str: ...
    @property
    def Exists(self) -> bool: ...
    @property
    def Key(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Parent(self) -> ResourceEntry: ...
    @Parent.setter
    def Parent(self, value: ResourceEntry) -> ResourceEntry: ...
    @property
    def Tags(self) -> SimTaxonomyEntryReferenceCollection: ...
    @property
    def UserWithWritingAccess(self) -> SimUserRole: ...
    @UserWithWritingAccess.setter
    def UserWithWritingAccess(self, value: SimUserRole) -> SimUserRole: ...
    @property
    def Visibility(self) -> SimComponentVisibility: ...
    @Visibility.setter
    def Visibility(self, value: SimComponentVisibility) -> SimComponentVisibility: ...
    def CanChangeLocation(self, _new_location: DirectoryInfo, nameCollisionFormat: str) -> ValueTuple_2[bool, str]: ...
    def CanChangePath(self, _new_data: FileSystemInfo, nameCollisionFormat: str) -> ValueTuple_2[bool, str]: ...
    def CanReplacePath(self, _replacement_path: str) -> bool: ...
    def ChangeLocation(self, _new_location: DirectoryInfo, nameCollisionFormat: str, _check_admissibility: bool) -> None: ...
    def ChangeName(self, _new_name: str, nameCollisionFormat: str) -> None: ...
    def ChangePath(self, _new_data: FileSystemInfo, nameCollisionFormat: str, _check_admissibility: bool) -> None: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, e1: ResourceEntry, e2: ResourceEntry) -> bool: ...
    def __ne__(self, e1: ResourceEntry, e2: ResourceEntry) -> bool: ...
    def RestoreDefaultTaxonomyReferences(self, projectData: ProjectData) -> None: ...
    def ToString(self) -> str: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, other: ResourceEntry) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...


    class ResourceChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class ResourceEntryDeletedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class ResourceEntryDeletingEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...



class ResourceFileEntry(ResourceEntry):
    @property
    def CanBeMoved(self) -> bool: ...
    @property
    def CanBeRenamed(self) -> bool: ...
    @property
    def CurrentFullPath(self) -> str: ...
    @CurrentFullPath.setter
    def CurrentFullPath(self, value: str) -> str: ...
    @property
    def CurrentRelativePath(self) -> str: ...
    @CurrentRelativePath.setter
    def CurrentRelativePath(self, value: str) -> str: ...
    @property
    def Exists(self) -> bool: ...
    @property
    def Extension(self) -> str: ...
    @property
    def Key(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def NameWithoutExtension(self) -> str: ...
    @property
    def Parent(self) -> ResourceEntry: ...
    @Parent.setter
    def Parent(self, value: ResourceEntry) -> ResourceEntry: ...
    @property
    def Tags(self) -> SimTaxonomyEntryReferenceCollection: ...
    @property
    def UserWithWritingAccess(self) -> SimUserRole: ...
    @UserWithWritingAccess.setter
    def UserWithWritingAccess(self, value: SimUserRole) -> SimUserRole: ...
    @property
    def Visibility(self) -> SimComponentVisibility: ...
    @Visibility.setter
    def Visibility(self, value: SimComponentVisibility) -> SimComponentVisibility: ...


class ResourceLocationError(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    OK : ResourceLocationError # 0
    CONTAINED_NOT_IN_WORKING_DIR : ResourceLocationError # 1
    LINKED_IN_WORKING_DIR : ResourceLocationError # 2
    LINKED_NOT_IN_FALLBACKS : ResourceLocationError # 4
    RESOURCE_IN_UNFAMILIAR_DIR : ResourceLocationError # 8
    RESOURCE_NOT_FOUND : ResourceLocationError # 16
    RESOURCE_CANNOT_BE_LOOKED_FOR : ResourceLocationError # 32




import typing
from System import SystemException, Exception
from System.Collections import IDictionary
from System.Reflection import MethodBase

class CryptographicException(SystemException):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, format: str, insert: str) -> None: ...
    @typing.overload
    def __init__(self, hr: int) -> None: ...
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str, inner: Exception) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...




import typing, abc
from System.Collections.Generic import IList_1, List_1

class CollectionUndoItem_GenericClasses(abc.ABCMeta):
    Generic_CollectionUndoItem_GenericClasses_CollectionUndoItem_1_T = typing.TypeVar('Generic_CollectionUndoItem_GenericClasses_CollectionUndoItem_1_T')
    def __getitem__(self, types : typing.Type[Generic_CollectionUndoItem_GenericClasses_CollectionUndoItem_1_T]) -> typing.Type[CollectionUndoItem_1[Generic_CollectionUndoItem_GenericClasses_CollectionUndoItem_1_T]]: ...

class CollectionUndoItem(CollectionUndoItem_0, metaclass =CollectionUndoItem_GenericClasses): ...

class CollectionUndoItem_0(abc.ABC):
    # Skipped Add due to it being static, abstract and generic.

    Add : Add_MethodGroup
    class Add_MethodGroup:
        def __getitem__(self, t:typing.Type[Add_1_T1]) -> Add_1[Add_1_T1]: ...

        Add_1_T1 = typing.TypeVar('Add_1_T1')
        class Add_1(typing.Generic[Add_1_T1]):
            Add_1_T = CollectionUndoItem_0.Add_MethodGroup.Add_1_T1
            def __call__(self, list: IList_1[Add_1_T], item: Add_1_T) -> CollectionUndoItem_1[Add_1_T]:...


    # Skipped Clear due to it being static, abstract and generic.

    Clear : Clear_MethodGroup
    class Clear_MethodGroup:
        def __getitem__(self, t:typing.Type[Clear_1_T1]) -> Clear_1[Clear_1_T1]: ...

        Clear_1_T1 = typing.TypeVar('Clear_1_T1')
        class Clear_1(typing.Generic[Clear_1_T1]):
            Clear_1_T = CollectionUndoItem_0.Clear_MethodGroup.Clear_1_T1
            def __call__(self, list: IList_1[Clear_1_T]) -> CollectionUndoItem_1[Clear_1_T]:...


    # Skipped Insert due to it being static, abstract and generic.

    Insert : Insert_MethodGroup
    class Insert_MethodGroup:
        def __getitem__(self, t:typing.Type[Insert_1_T1]) -> Insert_1[Insert_1_T1]: ...

        Insert_1_T1 = typing.TypeVar('Insert_1_T1')
        class Insert_1(typing.Generic[Insert_1_T1]):
            Insert_1_T = CollectionUndoItem_0.Insert_MethodGroup.Insert_1_T1
            def __call__(self, list: IList_1[Insert_1_T], item: Insert_1_T, idx: int) -> CollectionUndoItem_1[Insert_1_T]:...


    # Skipped Remove due to it being static, abstract and generic.

    Remove : Remove_MethodGroup
    class Remove_MethodGroup:
        def __getitem__(self, t:typing.Type[Remove_1_T1]) -> Remove_1[Remove_1_T1]: ...

        Remove_1_T1 = typing.TypeVar('Remove_1_T1')
        class Remove_1(typing.Generic[Remove_1_T1]):
            Remove_1_T = CollectionUndoItem_0.Remove_MethodGroup.Remove_1_T1
            def __call__(self, list: IList_1[Remove_1_T], item: Remove_1_T) -> CollectionUndoItem_1[Remove_1_T]:...


    # Skipped RemoveAt due to it being static, abstract and generic.

    RemoveAt : RemoveAt_MethodGroup
    class RemoveAt_MethodGroup:
        def __getitem__(self, t:typing.Type[RemoveAt_1_T1]) -> RemoveAt_1[RemoveAt_1_T1]: ...

        RemoveAt_1_T1 = typing.TypeVar('RemoveAt_1_T1')
        class RemoveAt_1(typing.Generic[RemoveAt_1_T1]):
            RemoveAt_1_T = CollectionUndoItem_0.RemoveAt_MethodGroup.RemoveAt_1_T1
            def __call__(self, list: IList_1[RemoveAt_1_T], idx: int) -> CollectionUndoItem_1[RemoveAt_1_T]:...




CollectionUndoItem_1_T = typing.TypeVar('CollectionUndoItem_1_T')
class CollectionUndoItem_1(typing.Generic[CollectionUndoItem_1_T], IUndoItem):
    def __init__(self, list: IList_1[CollectionUndoItem_1_T], action: UndoRedoAction, oldStartIndex: int, oldItems: IList_1[CollectionUndoItem_1_T], newStartIndex: int, newItems: IList_1[CollectionUndoItem_1_T]) -> None: ...
    def Execute(self) -> UndoExecutionResult: ...
    def Redo(self) -> None: ...
    def Undo(self) -> None: ...


class GroupUndoItem(IUndoItem):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, items: List_1[IUndoItem]) -> None: ...
    @property
    def Items(self) -> List_1[IUndoItem]: ...
    def Execute(self) -> UndoExecutionResult: ...
    def Redo(self) -> None: ...
    def Undo(self) -> None: ...


class IUndoItem(typing.Protocol):
    @abc.abstractmethod
    def Execute(self) -> UndoExecutionResult: ...
    @abc.abstractmethod
    def Redo(self) -> None: ...
    @abc.abstractmethod
    def Undo(self) -> None: ...


class PropertyUndoItem(IUndoItem):
    def __init__(self, target: typing.Any, property: str, value: typing.Any) -> None: ...
    def Execute(self) -> UndoExecutionResult: ...
    def Redo(self) -> None: ...
    def Undo(self) -> None: ...


class UndoExecutionResult(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Executed : UndoExecutionResult # 0
    PartiallyExecuted : UndoExecutionResult # 1
    Failed : UndoExecutionResult # 2


class UndoRedoAction(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Insert : UndoRedoAction # 0
    Remove : UndoRedoAction # 1
    Clear : UndoRedoAction # 2




import typing
from System.IO import FileInfo
from System.Collections.Generic import IEnumerable_1
from SIMULTAN.Data.Taxonomy import SimTaxonomy
from SIMULTAN.Projects import ExtendedProjectData
from SIMULTAN.Serializer.DXF import DXFStreamWriter, DXFParserInfo, DXFStreamReader

class SimTaxonomyDxfIO:
    def __init__(self) -> None: ...
    # Skipped Export due to it being static, abstract and generic.

    Export : Export_MethodGroup
    class Export_MethodGroup:
        @typing.overload
        def __call__(self, file: FileInfo, taxonomies: IEnumerable_1[SimTaxonomy], projectData: ExtendedProjectData) -> None:...
        @typing.overload
        def __call__(self, writer: DXFStreamWriter, taxonomies: IEnumerable_1[SimTaxonomy], projectData: ExtendedProjectData) -> None:...

    # Skipped Import due to it being static, abstract and generic.

    Import : Import_MethodGroup
    class Import_MethodGroup:
        @typing.overload
        def __call__(self, file: FileInfo, info: DXFParserInfo) -> None:...
        @typing.overload
        def __call__(self, reader: DXFStreamReader, parserInfo: DXFParserInfo) -> None:...

    # Skipped Read due to it being static, abstract and generic.

    Read : Read_MethodGroup
    class Read_MethodGroup:
        @typing.overload
        def __call__(self, file: FileInfo, info: DXFParserInfo) -> None:...
        @typing.overload
        def __call__(self, reader: DXFStreamReader, parserInfo: DXFParserInfo) -> None:...

    # Skipped Write due to it being static, abstract and generic.

    Write : Write_MethodGroup
    class Write_MethodGroup:
        @typing.overload
        def __call__(self, file: FileInfo, taxonomies: IEnumerable_1[SimTaxonomy], projectData: ExtendedProjectData) -> None:...
        @typing.overload
        def __call__(self, writer: DXFStreamWriter, taxonomies: IEnumerable_1[SimTaxonomy], projectData: ExtendedProjectData) -> None:...





import typing, abc
from System import IDisposable, DateTime, Decimal, TypeCode, Attribute, EventArgs, SystemException, Exception
from System.Collections import IDictionary, IEnumerator
from System.Reflection import MethodBase

class DeserializationToken(IDisposable):
    def Dispose(self) -> None: ...


class IDeserializationCallback(typing.Protocol):
    @abc.abstractmethod
    def OnDeserialization(self, sender: typing.Any) -> None: ...


class IFormatterConverter(typing.Protocol):
    @abc.abstractmethod
    def ToBoolean(self, value: typing.Any) -> bool: ...
    @abc.abstractmethod
    def ToByte(self, value: typing.Any) -> int: ...
    @abc.abstractmethod
    def ToChar(self, value: typing.Any) -> str: ...
    @abc.abstractmethod
    def ToDateTime(self, value: typing.Any) -> DateTime: ...
    @abc.abstractmethod
    def ToDecimal(self, value: typing.Any) -> Decimal: ...
    @abc.abstractmethod
    def ToDouble(self, value: typing.Any) -> float: ...
    @abc.abstractmethod
    def ToInt16(self, value: typing.Any) -> int: ...
    @abc.abstractmethod
    def ToInt32(self, value: typing.Any) -> int: ...
    @abc.abstractmethod
    def ToInt64(self, value: typing.Any) -> int: ...
    @abc.abstractmethod
    def ToSByte(self, value: typing.Any) -> int: ...
    @abc.abstractmethod
    def ToSingle(self, value: typing.Any) -> float: ...
    @abc.abstractmethod
    def ToString(self, value: typing.Any) -> str: ...
    @abc.abstractmethod
    def ToUInt16(self, value: typing.Any) -> int: ...
    @abc.abstractmethod
    def ToUInt32(self, value: typing.Any) -> int: ...
    @abc.abstractmethod
    def ToUInt64(self, value: typing.Any) -> int: ...
    # Skipped Convert due to it being static, abstract and generic.

    Convert : Convert_MethodGroup
    class Convert_MethodGroup:
        @typing.overload
        def __call__(self, value: typing.Any, typeCode: TypeCode) -> typing.Any:...
        @typing.overload
        def __call__(self, value: typing.Any, type: typing.Type[typing.Any]) -> typing.Any:...



class IObjectReference(typing.Protocol):
    @abc.abstractmethod
    def GetRealObject(self, context: StreamingContext) -> typing.Any: ...


class ISafeSerializationData(typing.Protocol):
    @abc.abstractmethod
    def CompleteDeserialization(self, deserialized: typing.Any) -> None: ...


class ISerializable(typing.Protocol):
    @abc.abstractmethod
    def GetObjectData(self, info: SerializationInfo, context: StreamingContext) -> None: ...


class OnDeserializedAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class OnDeserializingAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class OnSerializedAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class OnSerializingAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class OptionalFieldAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...
    @property
    def VersionAdded(self) -> int: ...
    @VersionAdded.setter
    def VersionAdded(self, value: int) -> int: ...


class SafeSerializationEventArgs(EventArgs):
    @property
    def StreamingContext(self) -> StreamingContext: ...
    def AddSerializedState(self, serializedState: ISafeSerializationData) -> None: ...


class SerializationEntry:
    @property
    def Name(self) -> str: ...
    @property
    def ObjectType(self) -> typing.Type[typing.Any]: ...
    @property
    def Value(self) -> typing.Any: ...


class SerializationException(SystemException):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str, innerException: Exception) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class SerializationInfo:
    @typing.overload
    def __init__(self, type: typing.Type[typing.Any], converter: IFormatterConverter) -> None: ...
    @typing.overload
    def __init__(self, type: typing.Type[typing.Any], converter: IFormatterConverter, requireSameTokenInPartialTrust: bool) -> None: ...
    @property
    def AssemblyName(self) -> str: ...
    @AssemblyName.setter
    def AssemblyName(self, value: str) -> str: ...
    @property
    def FullTypeName(self) -> str: ...
    @FullTypeName.setter
    def FullTypeName(self, value: str) -> str: ...
    @property
    def IsAssemblyNameSetExplicit(self) -> bool: ...
    @IsAssemblyNameSetExplicit.setter
    def IsAssemblyNameSetExplicit(self, value: bool) -> bool: ...
    @property
    def IsFullTypeNameSetExplicit(self) -> bool: ...
    @IsFullTypeNameSetExplicit.setter
    def IsFullTypeNameSetExplicit(self, value: bool) -> bool: ...
    @property
    def MemberCount(self) -> int: ...
    @property
    def ObjectType(self) -> typing.Type[typing.Any]: ...
    def GetBoolean(self, name: str) -> bool: ...
    def GetByte(self, name: str) -> int: ...
    def GetChar(self, name: str) -> str: ...
    def GetDateTime(self, name: str) -> DateTime: ...
    def GetDecimal(self, name: str) -> Decimal: ...
    def GetDouble(self, name: str) -> float: ...
    def GetEnumerator(self) -> SerializationInfoEnumerator: ...
    def GetInt16(self, name: str) -> int: ...
    def GetInt32(self, name: str) -> int: ...
    def GetInt64(self, name: str) -> int: ...
    def GetSByte(self, name: str) -> int: ...
    def GetSingle(self, name: str) -> float: ...
    def GetString(self, name: str) -> str: ...
    def GetUInt16(self, name: str) -> int: ...
    def GetUInt32(self, name: str) -> int: ...
    def GetUInt64(self, name: str) -> int: ...
    def GetValue(self, name: str, type: typing.Type[typing.Any]) -> typing.Any: ...
    def SetType(self, type: typing.Type[typing.Any]) -> None: ...
    @staticmethod
    def StartDeserialization() -> DeserializationToken: ...
    # Skipped AddValue due to it being static, abstract and generic.

    AddValue : AddValue_MethodGroup
    class AddValue_MethodGroup:
        @typing.overload
        def __call__(self, name: str, value: float) -> None:...
        # Method AddValue(name : String, value : Double) was skipped since it collides with above method
        @typing.overload
        def __call__(self, name: str, value: str) -> None:...
        # Method AddValue(name : String, value : SByte) was skipped since it collides with above method
        # Method AddValue(name : String, value : Byte) was skipped since it collides with above method
        # Method AddValue(name : String, value : Int16) was skipped since it collides with above method
        # Method AddValue(name : String, value : UInt16) was skipped since it collides with above method
        # Method AddValue(name : String, value : Int32) was skipped since it collides with above method
        # Method AddValue(name : String, value : UInt32) was skipped since it collides with above method
        # Method AddValue(name : String, value : Int64) was skipped since it collides with above method
        # Method AddValue(name : String, value : UInt64) was skipped since it collides with above method
        @typing.overload
        def __call__(self, name: str, value: Decimal) -> None:...
        # Method AddValue(name : String, value : Boolean) was skipped since it collides with above method
        @typing.overload
        def __call__(self, name: str, value: DateTime) -> None:...
        @typing.overload
        def __call__(self, name: str, value: typing.Any) -> None:...
        @typing.overload
        def __call__(self, name: str, value: typing.Any, type: typing.Type[typing.Any]) -> None:...



class SerializationInfoEnumerator(IEnumerator):
    @property
    def Current(self) -> SerializationEntry: ...
    @property
    def Name(self) -> str: ...
    @property
    def ObjectType(self) -> typing.Type[typing.Any]: ...
    @property
    def Value(self) -> typing.Any: ...
    def MoveNext(self) -> bool: ...
    def Reset(self) -> None: ...


class StreamingContext:
    @typing.overload
    def __init__(self, state: StreamingContextStates) -> None: ...
    @typing.overload
    def __init__(self, state: StreamingContextStates, additional: typing.Any) -> None: ...
    @property
    def Context(self) -> typing.Any: ...
    @property
    def State(self) -> StreamingContextStates: ...
    def Equals(self, obj: typing.Any) -> bool: ...
    def GetHashCode(self) -> int: ...


class StreamingContextStates(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    CrossProcess : StreamingContextStates # 1
    CrossMachine : StreamingContextStates # 2
    File : StreamingContextStates # 4
    Persistence : StreamingContextStates # 8
    Remoting : StreamingContextStates # 16
    Other : StreamingContextStates # 32
    Clone : StreamingContextStates # 64
    CrossAppDomain : StreamingContextStates # 128
    All : StreamingContextStates # 255




from datetime import datetime
import os
import numpy as np
import pandas as pd
from PySimultan2.data_model import DataModel
from PySimultan2.multi_values import *
from PySimultan2.utils import get_or_create_taxonomy_entry
from PySimultan2.object_mapper import PythonMapper
from PySimultan2.taxonomy_maps import TaxonomyMap, Content

from SIMULTAN.Utils import IntIndex3D
from SIMULTAN.Data.SimMath import SimPoint3D

import resources

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 importlib_resources.
    import importlib_resources as pkg_resources


with pkg_resources.path(resources, 'test_multi_value_field_project.simultan') as r_path:
    project_path = str(r_path)

data_model = DataModel.create_new_project(project_path=project_path,
                                          user_name='admin',
                                          password='admin')


def test_value_field_creation():
    numpy_array = np.arange(1000, dtype=float).reshape(10, 10, 10)
    field = numpy_to_simultan_multi_value_field_3d(numpy_array, 'test_field')
    array = simultan_multi_value_field_3d_to_numpy(field)

    assert np.allclose(numpy_array, array)


def test_pandas_value_field_creation():
    df = pd.DataFrame(np.arange(24, dtype=float).reshape(6, 4),
                      columns=list("ABCD"))
    field = pandas_to_simultan_multi_value_big_table(df, 'test_field')
    loaded_df = simultan_multi_value_big_table_to_pandas(field)

    assert np.allclose(df.values, loaded_df.values)


def test_parameter_creation():
    tax_entry = get_or_create_taxonomy_entry(name='test_slot', key='test_slot', data_model=data_model)
    numpy_array = np.random.rand(10, 10, 10)
    parameter = create_field_parameter(name='test_parameter',
                                       value=numpy_array,
                                       slot=tax_entry,
                                       data_model=data_model)
    assert np.allclose(numpy_array, simultan_multi_value_field_3d_to_numpy(parameter.ValueSource.Field))

# data_model.data[0].Parameters[0].ValueSource.Field.GetValue(SimPoint3D(0, 0, 0))


def test_typed_data_model():

    mapper = PythonMapper()
    data = data_model.get_typed_data(mapper)
    assert data[0]._parameters


def test_set_parameter_to_value_field():

    # create a new component
    mapper = PythonMapper()

    class TestComponent(object):
        def __init__(self, *args, **kwargs):
            self.numpy_value = kwargs.get('numpy_value')
            self.pandas_value = kwargs.get('pandas_value')

    content0 = Content(text_or_key='numpy_value',  # text or key of the content/parameter/property
                       property_name='numpy_value',  # name of the generated property
                       type=np.ndarray,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='numpy_value to test')

    content1 = Content(text_or_key='pandas_value',  # text or key of the content/parameter/property
                       property_name='pandas_value',  # name of the generated property
                       type=np.ndarray,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='pandas_value to test')

    test_component_map = TaxonomyMap(taxonomy_name='Test',
                                     taxonomy_key='Test',
                                     taxonomy_entry_name='TestComponent',
                                     taxonomy_entry_key='TestComponent',
                                     content=[content0, content1],
                                     )

    mapper.register(test_component_map.taxonomy_entry_key, TestComponent, taxonomy_map=test_component_map)
    mapped_test_component_cls = mapper.get_mapped_class(test_component_map.taxonomy_entry_key)

    df = pd.DataFrame(np.random.rand(6, 4),
                      columns=list("ABCD"))

    test_component = mapped_test_component_cls(name='test_component',
                                               numpy_value=np.random.rand(10, 10, 10),
                                               pandas_value=df)

    test_component.numpy_value = np.arange(1000, dtype=float).reshape(10, 10, 10)
    test_component.pandas_value = pd.DataFrame(np.arange(24, dtype=float).reshape(6, 4),
                                               columns=list("ABCD"))

    data_model.save()


if __name__ == '__main__':
    test_value_field_creation()
    test_pandas_value_field_creation()
    test_parameter_creation()
    test_set_parameter_to_value_field()
    test_typed_data_model()


data_model.cleanup()
print('Test passed')



import typing, abc
from System import Uri, Array_1
from System.Security import SecureString
from System.IO import TextWriter

class ICredentials(typing.Protocol):
    @abc.abstractmethod
    def GetCredential(self, uri: Uri, authType: str) -> NetworkCredential: ...


class ICredentialsByHost(typing.Protocol):
    @abc.abstractmethod
    def GetCredential(self, host: str, port: int, authenticationType: str) -> NetworkCredential: ...


class NetworkCredential(ICredentialsByHost, ICredentials):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, userName: str, password: str) -> None: ...
    @typing.overload
    def __init__(self, userName: str, password: SecureString) -> None: ...
    @typing.overload
    def __init__(self, userName: str, password: str, domain: str) -> None: ...
    @typing.overload
    def __init__(self, userName: str, password: SecureString, domain: str) -> None: ...
    @property
    def Domain(self) -> str: ...
    @Domain.setter
    def Domain(self, value: str) -> str: ...
    @property
    def Password(self) -> str: ...
    @Password.setter
    def Password(self, value: str) -> str: ...
    @property
    def SecurePassword(self) -> SecureString: ...
    @SecurePassword.setter
    def SecurePassword(self, value: SecureString) -> SecureString: ...
    @property
    def UserName(self) -> str: ...
    @UserName.setter
    def UserName(self, value: str) -> str: ...
    # Skipped GetCredential due to it being static, abstract and generic.

    GetCredential : GetCredential_MethodGroup
    class GetCredential_MethodGroup:
        @typing.overload
        def __call__(self, uri: Uri, authenticationType: str) -> NetworkCredential:...
        @typing.overload
        def __call__(self, host: str, port: int, authenticationType: str) -> NetworkCredential:...



class WebUtility(abc.ABC):
    @staticmethod
    def UrlDecode(encodedValue: str) -> str: ...
    @staticmethod
    def UrlDecodeToBytes(encodedValue: Array_1[int], offset: int, count: int) -> Array_1[int]: ...
    @staticmethod
    def UrlEncode(value: str) -> str: ...
    @staticmethod
    def UrlEncodeToBytes(value: Array_1[int], offset: int, count: int) -> Array_1[int]: ...
    # Skipped HtmlDecode due to it being static, abstract and generic.

    HtmlDecode : HtmlDecode_MethodGroup
    class HtmlDecode_MethodGroup:
        @typing.overload
        def __call__(self, value: str) -> str:...
        @typing.overload
        def __call__(self, value: str, output: TextWriter) -> None:...

    # Skipped HtmlEncode due to it being static, abstract and generic.

    HtmlEncode : HtmlEncode_MethodGroup
    class HtmlEncode_MethodGroup:
        @typing.overload
        def __call__(self, value: str) -> str:...
        @typing.overload
        def __call__(self, value: str, output: TextWriter) -> None:...





import typing, clr, abc
from System.Buffers import OperationStatus, StandardFormat
from System import ReadOnlySpan_1, Span_1, Decimal, DateTimeOffset, DateTime, Guid, TimeSpan

class Base64(abc.ABC):
    @staticmethod
    def DecodeFromUtf8(utf8: ReadOnlySpan_1[int], bytes: Span_1[int], bytesConsumed: clr.Reference[int], bytesWritten: clr.Reference[int], isFinalBlock: bool = ...) -> OperationStatus: ...
    @staticmethod
    def DecodeFromUtf8InPlace(buffer: Span_1[int], bytesWritten: clr.Reference[int]) -> OperationStatus: ...
    @staticmethod
    def EncodeToUtf8(bytes: ReadOnlySpan_1[int], utf8: Span_1[int], bytesConsumed: clr.Reference[int], bytesWritten: clr.Reference[int], isFinalBlock: bool = ...) -> OperationStatus: ...
    @staticmethod
    def EncodeToUtf8InPlace(buffer: Span_1[int], dataLength: int, bytesWritten: clr.Reference[int]) -> OperationStatus: ...
    @staticmethod
    def GetMaxDecodedFromUtf8Length(length: int) -> int: ...
    @staticmethod
    def GetMaxEncodedToUtf8Length(length: int) -> int: ...
    # Skipped IsValid due to it being static, abstract and generic.

    IsValid : IsValid_MethodGroup
    class IsValid_MethodGroup:
        @typing.overload
        def __call__(self, base64Text: ReadOnlySpan_1[str]) -> bool:...
        @typing.overload
        def __call__(self, base64TextUtf8: ReadOnlySpan_1[int]) -> bool:...
        @typing.overload
        def __call__(self, base64TextUtf8: ReadOnlySpan_1[int], decodedLength: clr.Reference[int]) -> bool:...
        @typing.overload
        def __call__(self, base64Text: ReadOnlySpan_1[str], decodedLength: clr.Reference[int]) -> bool:...



class Utf8Formatter(abc.ABC):
    # Skipped TryFormat due to it being static, abstract and generic.

    TryFormat : TryFormat_MethodGroup
    class TryFormat_MethodGroup:
        @typing.overload
        def __call__(self, value: float, destination: Span_1[int], bytesWritten: clr.Reference[int], format: StandardFormat = ...) -> bool:...
        # Method TryFormat(value : Single, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        # Method TryFormat(value : Byte, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        # Method TryFormat(value : SByte, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        # Method TryFormat(value : UInt16, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        # Method TryFormat(value : Int16, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        # Method TryFormat(value : UInt32, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        # Method TryFormat(value : Int32, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        # Method TryFormat(value : UInt64, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        # Method TryFormat(value : Int64, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: Decimal, destination: Span_1[int], bytesWritten: clr.Reference[int], format: StandardFormat = ...) -> bool:...
        # Method TryFormat(value : Boolean, destination : Span`1, bytesWritten : Int32&, format : StandardFormat) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: DateTimeOffset, destination: Span_1[int], bytesWritten: clr.Reference[int], format: StandardFormat = ...) -> bool:...
        @typing.overload
        def __call__(self, value: DateTime, destination: Span_1[int], bytesWritten: clr.Reference[int], format: StandardFormat = ...) -> bool:...
        @typing.overload
        def __call__(self, value: Guid, destination: Span_1[int], bytesWritten: clr.Reference[int], format: StandardFormat = ...) -> bool:...
        @typing.overload
        def __call__(self, value: TimeSpan, destination: Span_1[int], bytesWritten: clr.Reference[int], format: StandardFormat = ...) -> bool:...



class Utf8Parser(abc.ABC):
    # Skipped TryParse due to it being static, abstract and generic.

    TryParse : TryParse_MethodGroup
    class TryParse_MethodGroup:
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], value: clr.Reference[float], bytesConsumed: clr.Reference[int], standardFormat: str = ...) -> bool:...
        # Method TryParse(source : ReadOnlySpan`1, value : Double&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        # Method TryParse(source : ReadOnlySpan`1, value : SByte&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        # Method TryParse(source : ReadOnlySpan`1, value : Int16&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        # Method TryParse(source : ReadOnlySpan`1, value : Int32&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        # Method TryParse(source : ReadOnlySpan`1, value : Int64&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        # Method TryParse(source : ReadOnlySpan`1, value : Byte&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        # Method TryParse(source : ReadOnlySpan`1, value : UInt16&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        # Method TryParse(source : ReadOnlySpan`1, value : UInt32&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        # Method TryParse(source : ReadOnlySpan`1, value : UInt64&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], value: clr.Reference[Decimal], bytesConsumed: clr.Reference[int], standardFormat: str = ...) -> bool:...
        # Method TryParse(source : ReadOnlySpan`1, value : Boolean&, bytesConsumed : Int32&, standardFormat : Char) was skipped since it collides with above method
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], value: clr.Reference[DateTime], bytesConsumed: clr.Reference[int], standardFormat: str = ...) -> bool:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], value: clr.Reference[DateTimeOffset], bytesConsumed: clr.Reference[int], standardFormat: str = ...) -> bool:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], value: clr.Reference[Guid], bytesConsumed: clr.Reference[int], standardFormat: str = ...) -> bool:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], value: clr.Reference[TimeSpan], bytesConsumed: clr.Reference[int], standardFormat: str = ...) -> bool:...





from PySimultan2 import FileInfo
from PySimultan2 import DataModel
from PySimultan2 import PythonMapper
from PySimultan2.taxonomy_maps import TaxonomyMap, Content
from PySimultan2.simultan_object import SimultanObject

from PySimultan2.tests import resources

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 importlib_resources.
    import importlib_resources as pkg_resources


with pkg_resources.path(resources, 'test_file_load.simultan') as r_path:
    project_path = str(r_path)

data_model = DataModel(project_path=project_path,
                       user_name='admin',
                       password='admin')


def test_load_file_component():

    # create a new component
    mapper = PythonMapper()

    class TestComponent(object):
        def __init__(self, *args, **kwargs):
            self.file_content_1 = kwargs.get('file_content_1')
            self.file_content_2 = kwargs.get('file_content_2')

    content0 = Content(text_or_key='file_content_1',  # text or key of the content/parameter/property
                       property_name='file_content_1',  # name of the generated property
                       type='file',  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='file_value to test')

    content1 = Content(text_or_key='file_content_2',  # text or key of the content/parameter/property
                       property_name='file_content_2',  # name of the generated property
                       type='file',  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='file_value_2 to test')

    content2 = Content(text_or_key='param_1',  # text or key of the content/parameter/property
                       property_name='param_1',  # name of the generated property
                       type=None,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='param_1 to test')

    content3 = Content(text_or_key='param_2',  # text or key of the content/parameter/property
                       property_name='param_2',  # name of the generated property
                       type=None,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='param_2 to test')

    content4 = Content(text_or_key='sub_component',  # text or key of the content/parameter/property
                       property_name='sub_component',  # name of the generated property
                       type=None,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='sub_component to test')

    content5 = Content(text_or_key='referenced_component',  # text or key of the content/parameter/property
                       property_name='referenced_component',  # name of the generated property
                       type=None,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='referenced_component to test')

    test_component_map = TaxonomyMap(taxonomy_name='PySimultan',
                                     taxonomy_key='Test',
                                     taxonomy_entry_name='test_component',
                                     taxonomy_entry_key='test_component',
                                     content=[content0, content1, content2, content3, content4, content5],
                                     )

    mapper.register(test_component_map.taxonomy_entry_key, TestComponent, taxonomy_map=test_component_map)
    mapped_test_component_cls = mapper.get_mapped_class(test_component_map.taxonomy_entry_key)

    data = data_model.get_typed_data(mapper)
    assert type(data[0].file_content_1) is FileInfo
    assert data[0].param_2 == 10
    assert isinstance(data[0].sub_component, SimultanObject)

    print(data[0].file_content_1.content)


if __name__ == '__main__':
    print(data_model.data[0].ReferencedAssets[0].ResourceKey)
    print(data_model.data[0].ReferencedAssets[0].Resource)
    print(data_model.data[0].ReferencedAssets[0].Resource.Name)
    print(data_model.data[0].ReferencedAssets[0].Resource.File.FullPath)

    test_load_file_component()

data_model.cleanup()
print('Test passed')



import typing, abc
from System import Guid, ValueTuple_2, Predicate_1, MulticastDelegate, IAsyncResult, AsyncCallback
from System.IO import FileInfo, DirectoryInfo
from SIMULTAN.Projects.ManagedFiles import ManagedFileCollection, ManagedFile
from System.Collections.Generic import IEnumerable_1, List_1, IReadOnlyList_1, Dictionary_2
from System.ComponentModel import ISynchronizeInvoke
from SIMULTAN.Utils import IDispatcherTimerFactory
from SIMULTAN.Data.Assets import AssetManager, MultiLinkManager, ResourceFileEntry, ResourceLocationError, ResourceEntry, ResourceDirectoryEntry, LinkedResourceFileEntry, ContainedResourceFileEntry
from SIMULTAN.Exchange import ComponentGeometryExchange
from SIMULTAN.Data.Components import SimComponentCollection, ParameterFactory, SimUserComponentListCollection, SimCalculation, SimComponent, SimBaseParameter
from SIMULTAN.DataMapping import SimDataMappingToolCollection
from SIMULTAN.Data.Geometry import SimGeometryModelCollection, SimGeometryRelationCollection
from SIMULTAN.Data import SimIdGenerator, IReferenceLocation
from SIMULTAN.Data.FlowNetworks import SimNetworkFactory
from SIMULTAN.Data.SimNetworks import SimNetwork
from SIMULTAN.Data.SitePlanner import SitePlannerManager
from SIMULTAN.Data.Taxonomy import SimTaxonomyCollection
from SIMULTAN.Data.Users import SimUsersManager, SimUser
from SIMULTAN.Data.MultiValues import SimMultiValueCollection, SimMultiValue
from SIMULTAN.Data.ValueMappings import SimValueMappingCollection
from System.Collections.ObjectModel import ObservableCollection_1
from System.Reflection import MethodInfo

class CompactProject(HierarchicalProject):
    def __init__(self, id: Guid, _project_file: FileInfo, _all_managers: ExtendedProjectData, _files: ManagedFileCollection, _non_managed_files: IEnumerable_1[FileInfo], _contained_dirs: IEnumerable_1[DirectoryInfo], _associated_files: IEnumerable_1[FileInfo], _unpack_folder: DirectoryInfo) -> None: ...
    @property
    def AllProjectDataManagers(self) -> ExtendedProjectData: ...
    @AllProjectDataManagers.setter
    def AllProjectDataManagers(self, value: ExtendedProjectData) -> ExtendedProjectData: ...
    @property
    def AssociatedFilesCount(self) -> int: ...
    @property
    def AuthenticationSkipped(self) -> bool: ...
    @AuthenticationSkipped.setter
    def AuthenticationSkipped(self, value: bool) -> bool: ...
    @property
    def Children(self) -> HierarchicalProject.ChildProjectCollection: ...
    @property
    def ContainedDirectoriesCopy(self) -> List_1[DirectoryInfo]: ...
    @property
    def ContainsUnsavedChanges(self) -> bool: ...
    @ContainsUnsavedChanges.setter
    def ContainsUnsavedChanges(self, value: bool) -> bool: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def IsAuthenticated(self) -> bool: ...
    @property
    def IsLoaded(self) -> bool: ...
    @property
    def IsOpened(self) -> bool: ...
    @property
    def IsReadyForAuthentication(self) -> bool: ...
    @property
    def ManagedFiles(self) -> ManagedFileCollection: ...
    @ManagedFiles.setter
    def ManagedFiles(self, value: ManagedFileCollection) -> ManagedFileCollection: ...
    @property
    def Name(self) -> str: ...
    @property
    def NonManagedFiles(self) -> IReadOnlyList_1[FileInfo]: ...
    @property
    def ParentProjects(self) -> IReadOnlyList_1[HierarchicalProject]: ...
    @property
    def ProjectFile(self) -> FileInfo: ...
    @property
    def ProjectUnpackFolder(self) -> DirectoryInfo: ...
    @ProjectUnpackFolder.setter
    def ProjectUnpackFolder(self, value: DirectoryInfo) -> DirectoryInfo: ...
    def UpdateBackupFile(self) -> None: ...


class ExtendedProjectData(ProjectData):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, synchronizationContext: ISynchronizeInvoke, dispatcherTimer: IDispatcherTimerFactory) -> None: ...
    @property
    def AssetManager(self) -> AssetManager: ...
    @property
    def ComponentGeometryExchange(self) -> ComponentGeometryExchange: ...
    @property
    def Components(self) -> SimComponentCollection: ...
    @property
    def DataMappingTools(self) -> SimDataMappingToolCollection: ...
    @property
    def DispatcherTimerFactory(self) -> IDispatcherTimerFactory: ...
    @DispatcherTimerFactory.setter
    def DispatcherTimerFactory(self, value: IDispatcherTimerFactory) -> IDispatcherTimerFactory: ...
    @property
    def GeometryModels(self) -> SimGeometryModelCollection: ...
    @property
    def GeometryRelations(self) -> SimGeometryRelationCollection: ...
    @property
    def IdGenerator(self) -> SimIdGenerator: ...
    @property
    def ImportLogFile(self) -> FileInfo: ...
    @ImportLogFile.setter
    def ImportLogFile(self, value: FileInfo) -> FileInfo: ...
    @property
    def MultiLinkManager(self) -> MultiLinkManager: ...
    @property
    def NetworkManager(self) -> SimNetworkFactory: ...
    @property
    def Owner(self) -> IReferenceLocation: ...
    @Owner.setter
    def Owner(self, value: IReferenceLocation) -> IReferenceLocation: ...
    @property
    def ParameterLibraryManager(self) -> ParameterFactory: ...
    @property
    def Project(self) -> HierarchicalProject: ...
    @Project.setter
    def Project(self, value: HierarchicalProject) -> HierarchicalProject: ...
    @property
    def SimNetworks(self) -> SimNetwork.SimNetworkCollection: ...
    @property
    def SitePlannerManager(self) -> SitePlannerManager: ...
    @property
    def SynchronizationContext(self) -> ISynchronizeInvoke: ...
    @SynchronizationContext.setter
    def SynchronizationContext(self, value: ISynchronizeInvoke) -> ISynchronizeInvoke: ...
    @property
    def Taxonomies(self) -> SimTaxonomyCollection: ...
    @property
    def UserComponentLists(self) -> SimUserComponentListCollection: ...
    @property
    def UsersManager(self) -> SimUsersManager: ...
    @property
    def ValueManager(self) -> SimMultiValueCollection: ...
    @property
    def ValueMappings(self) -> SimValueMappingCollection: ...
    def Reset(self) -> None: ...


class HierarchicalProject(IReferenceLocation, abc.ABC):
    @property
    def AllProjectDataManagers(self) -> ExtendedProjectData: ...
    @AllProjectDataManagers.setter
    def AllProjectDataManagers(self, value: ExtendedProjectData) -> ExtendedProjectData: ...
    @property
    def AssociatedFilesCount(self) -> int: ...
    @property
    def AuthenticationSkipped(self) -> bool: ...
    @AuthenticationSkipped.setter
    def AuthenticationSkipped(self, value: bool) -> bool: ...
    @property
    def Children(self) -> HierarchicalProject.ChildProjectCollection: ...
    @property
    def ContainedDirectoriesCopy(self) -> List_1[DirectoryInfo]: ...
    @property
    def ContainsUnsavedChanges(self) -> bool: ...
    @ContainsUnsavedChanges.setter
    def ContainsUnsavedChanges(self, value: bool) -> bool: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def IsAuthenticated(self) -> bool: ...
    @IsAuthenticated.setter
    def IsAuthenticated(self, value: bool) -> bool: ...
    @property
    def IsLoaded(self) -> bool: ...
    @IsLoaded.setter
    def IsLoaded(self, value: bool) -> bool: ...
    @property
    def IsOpened(self) -> bool: ...
    @IsOpened.setter
    def IsOpened(self, value: bool) -> bool: ...
    @property
    def IsReadyForAuthentication(self) -> bool: ...
    @IsReadyForAuthentication.setter
    def IsReadyForAuthentication(self, value: bool) -> bool: ...
    @property
    def ManagedFiles(self) -> ManagedFileCollection: ...
    @ManagedFiles.setter
    def ManagedFiles(self, value: ManagedFileCollection) -> ManagedFileCollection: ...
    @property
    def Name(self) -> str: ...
    @property
    def NonManagedFiles(self) -> IReadOnlyList_1[FileInfo]: ...
    @property
    def ParentProjects(self) -> IReadOnlyList_1[HierarchicalProject]: ...
    @property
    def ProjectUnpackFolder(self) -> DirectoryInfo: ...
    @ProjectUnpackFolder.setter
    def ProjectUnpackFolder(self, value: DirectoryInfo) -> DirectoryInfo: ...
    def AddEmptyGeoMapResource(self, _target: DirectoryInfo, _initial_file_name: str, nameCollisionFormat: str) -> ResourceFileEntry: ...
    def AddEmptyGeometryResource(self, _target: DirectoryInfo, _initial_file_name: str, nameCollisionFormat: str, dispatcherTimer: IDispatcherTimerFactory) -> ResourceFileEntry: ...
    def AddEmptyGeometryResourceWithoutCollisionCheck(self, _target: DirectoryInfo, file_name: str) -> ResourceFileEntry: ...
    def AddEmptyResource(self, file: FileInfo) -> ResourceFileEntry: ...
    def AddEmptySitePlannerResource(self, _target: DirectoryInfo, _initial_file_name: str, nameCollisionFormat: str) -> ResourceFileEntry: ...
    def AddResourceFile(self, file: FileInfo, projectDataManager: ExtendedProjectData) -> None: ...
    def AddUndifferentiated(self, _files: IEnumerable_1[FileInfo]) -> None: ...
    def CanLinkAsResource(self, _file: FileInfo) -> ResourceLocationError: ...
    def CanMoveResourceEntry(self, _resource: ResourceEntry, _target: ResourceDirectoryEntry, nameCollisionFormat: str) -> ValueTuple_2[bool, str]: ...
    def CanRenameResourceEntry(self, _resource: ResourceEntry, _name: str, nameCollisionFormat: str, _name_is_full_name: bool = ...) -> ValueTuple_2[bool, str]: ...
    def ContainedToLinked(self, _original: ContainedResourceFileEntry, _target_folder: DirectoryInfo, copyNameFormat: str) -> LinkedResourceFileEntry: ...
    def CopyResource(self, source: ResourceEntry, _target: ResourceDirectoryEntry, nameCollisionFormat: str, newOwner: SimUser = ...) -> None: ...
    def CopyResourceAsContainedFileEntry(self, file: FileInfo, targetDir: DirectoryInfo, copyNameFormat: str) -> ResourceEntry: ...
    def CreateResourceDirIn(self, _dir_name: str, _target_dir: DirectoryInfo, collisionNameFormat: str) -> ResourceEntry: ...
    def DeleteResource(self, _resource_to_delete: ResourceEntry) -> ValueTuple_2[bool, bool]: ...
    def DisableProjectUnpackFolderWatcher(self) -> None: ...
    def EnableProjectUnpackFolderWatcher(self) -> None: ...
    def FindProjects(self, predicate: Predicate_1[HierarchicalProject]) -> IEnumerable_1[HierarchicalProject]: ...
    def GetAssociatedFiles(self) -> None: ...
    def GetCorresponding(self, _resource: ResourceEntry) -> ManagedFile: ...
    def LinkedToContained(self, _original: LinkedResourceFileEntry, nameFormat: str) -> ContainedResourceFileEntry: ...
    def LinkResourceFile(self, _file: FileInfo, _target_dir: DirectoryInfo, _allow_duplicates: bool) -> ValueTuple_2[ResourceEntry, bool]: ...
    @staticmethod
    def LoadDefaultTaxonomies(projectData: ExtendedProjectData) -> bool: ...
    def MoveResourceEntry(self, _resource: ResourceEntry, _target: ResourceDirectoryEntry, nameCollisionFormat: str, _check_admissibility: bool) -> None: ...
    def OnFileImportTimeout(self, file: FileInfo) -> None: ...
    def OnProjectAuthenticated(self, _successfully: bool) -> None: ...
    def OnProjectClosed(self) -> None: ...
    def OnProjectLoaded(self) -> None: ...
    def OnProjectOpened(self) -> None: ...
    def OnProjectReadyForAuthentication(self) -> None: ...
    def OnProjectUnloaded(self) -> None: ...
    def OnProjectUpToDateChanged(self, isUpToDate: bool) -> None: ...
    def PassNonManagedAndAllDirs(self, _non_managed_files: IEnumerable_1[FileInfo], _contained_dirs: IEnumerable_1[DirectoryInfo]) -> None: ...
    def RenameResourceEntry(self, _resource: ResourceEntry, _name: str, nameCollisionFormat: str) -> None: ...
    def ReplaceLinkedResource(self, _resource: ResourceEntry, _new_file: FileInfo, _allow_duplicates: bool) -> bool: ...
    def ReplaceManagedFiles(self, _old_files: IEnumerable_1[FileInfo], _new_files: IEnumerable_1[FileInfo]) -> None: ...
    def ResetNonManaged(self) -> None: ...
    # Skipped IsInProject due to it being static, abstract and generic.

    IsInProject : IsInProject_MethodGroup
    class IsInProject_MethodGroup:
        @typing.overload
        def __call__(self, _calculation: SimCalculation) -> bool:...
        @typing.overload
        def __call__(self, _component: SimComponent) -> bool:...
        @typing.overload
        def __call__(self, multiValue: SimMultiValue) -> bool:...
        @typing.overload
        def __call__(self, _parameter: SimBaseParameter) -> bool:...
        @typing.overload
        def __call__(self, resource: ResourceEntry) -> bool:...


    class ChildProjectCollection(ObservableCollection_1[HierarchicalProject]):
        def __init__(self, owner: HierarchicalProject) -> None: ...
        @property
        def Count(self) -> int: ...
        @property
        def Item(self) -> HierarchicalProject: ...
        @Item.setter
        def Item(self, value: HierarchicalProject) -> HierarchicalProject: ...


    class FileImportTimeoutEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, file: FileInfo, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, file: FileInfo) -> None: ...


    class ProjectAuthenticatedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class ProjectClosedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class ProjectLoadedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class ProjectOpenedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class ProjectReadyForAuthenticationEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class ProjectUnloadedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class ProjectUpToDateChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, isUpToDate: bool, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, isUpToDate: bool) -> None: ...



class HierarchicProjectMetaData:
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, _project_id: Guid, _children: Dictionary_2[Guid, str]) -> None: ...
    @typing.overload
    def __init__(self, original: HierarchicProjectMetaData) -> None: ...
    @property
    def ChildProjects(self) -> Dictionary_2[Guid, str]: ...
    @property
    def ProjectId(self) -> Guid: ...
    def AddChildProject(self, _project_id: Guid, _project_folder: DirectoryInfo, _child_project_folder: DirectoryInfo) -> None: ...
    def RemoveChildProject(self, _project_id: Guid) -> None: ...


class ProjectData(abc.ABC):
    def __init__(self, synchronizationContext: ISynchronizeInvoke, dispatcherTimer: IDispatcherTimerFactory) -> None: ...
    @property
    def AssetManager(self) -> AssetManager: ...
    @property
    def ComponentGeometryExchange(self) -> ComponentGeometryExchange: ...
    @property
    def Components(self) -> SimComponentCollection: ...
    @property
    def DataMappingTools(self) -> SimDataMappingToolCollection: ...
    @property
    def DispatcherTimerFactory(self) -> IDispatcherTimerFactory: ...
    @DispatcherTimerFactory.setter
    def DispatcherTimerFactory(self, value: IDispatcherTimerFactory) -> IDispatcherTimerFactory: ...
    @property
    def GeometryModels(self) -> SimGeometryModelCollection: ...
    @property
    def GeometryRelations(self) -> SimGeometryRelationCollection: ...
    @property
    def IdGenerator(self) -> SimIdGenerator: ...
    @property
    def ImportLogFile(self) -> FileInfo: ...
    @ImportLogFile.setter
    def ImportLogFile(self, value: FileInfo) -> FileInfo: ...
    @property
    def MultiLinkManager(self) -> MultiLinkManager: ...
    @property
    def NetworkManager(self) -> SimNetworkFactory: ...
    @property
    def Owner(self) -> IReferenceLocation: ...
    @Owner.setter
    def Owner(self, value: IReferenceLocation) -> IReferenceLocation: ...
    @property
    def ParameterLibraryManager(self) -> ParameterFactory: ...
    @property
    def SimNetworks(self) -> SimNetwork.SimNetworkCollection: ...
    @property
    def SitePlannerManager(self) -> SitePlannerManager: ...
    @property
    def SynchronizationContext(self) -> ISynchronizeInvoke: ...
    @SynchronizationContext.setter
    def SynchronizationContext(self, value: ISynchronizeInvoke) -> ISynchronizeInvoke: ...
    @property
    def Taxonomies(self) -> SimTaxonomyCollection: ...
    @property
    def UserComponentLists(self) -> SimUserComponentListCollection: ...
    @property
    def UsersManager(self) -> SimUsersManager: ...
    @property
    def ValueManager(self) -> SimMultiValueCollection: ...
    @property
    def ValueMappings(self) -> SimValueMappingCollection: ...
    def Clear(self) -> None: ...
    def RestoreDefaultTaxonomyReferences(self, taxonomyFileVersion: int = ...) -> None: ...
    def SetCallingLocation(self, caller: IReferenceLocation) -> None: ...


class ResourceReference:
    @typing.overload
    def __init__(self, projectId: Guid, res: ResourceFileEntry) -> None: ...
    @typing.overload
    def __init__(self, projectId: Guid, resourceIndex: int) -> None: ...
    @typing.overload
    def __init__(self, projectId: Guid, resourceIndex: int, assetManager: AssetManager) -> None: ...
    @property
    def Exists(self) -> bool: ...
    @property
    def ProjectId(self) -> Guid: ...
    @ProjectId.setter
    def ProjectId(self, value: Guid) -> Guid: ...
    @property
    def ResourceFile(self) -> ResourceFileEntry: ...
    @ResourceFile.setter
    def ResourceFile(self, value: ResourceFileEntry) -> ResourceFileEntry: ...
    @property
    def ResourceIndex(self) -> int: ...
    @ResourceIndex.setter
    def ResourceIndex(self, value: int) -> int: ...
    def Equals(self, obj: typing.Any) -> bool: ...
    def GetHashCode(self) -> int: ...
    def LazyLoad(self, assetManager: AssetManager) -> None: ...




import typing, abc
from System.Collections import IComparer, IEqualityComparer, IEnumerable, IEnumerator
from System.Xml import IXmlNamespaceResolver, XmlNamespaceManager, XmlNameTable, XmlNodeOrder, XmlWriter, XmlNamespaceScope, XmlReader
from System import DateTime, ICloneable
from System.Xml.Schema import XmlSchemaType, IXmlSchemaInfo, XmlSchemaSet, ValidationEventHandler
from System.Collections.Generic import IDictionary_2

class IXPathNavigable(typing.Protocol):
    @abc.abstractmethod
    def CreateNavigator(self) -> XPathNavigator: ...


class XmlCaseOrder(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : XmlCaseOrder # 0
    UpperFirst : XmlCaseOrder # 1
    LowerFirst : XmlCaseOrder # 2


class XmlDataType(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Text : XmlDataType # 1
    Number : XmlDataType # 2


class XmlSortOrder(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Ascending : XmlSortOrder # 1
    Descending : XmlSortOrder # 2


class XPathExpression(abc.ABC):
    @property
    def Expression(self) -> str: ...
    @property
    def ReturnType(self) -> XPathResultType: ...
    @abc.abstractmethod
    def Clone(self) -> XPathExpression: ...
    # Skipped AddSort due to it being static, abstract and generic.

    AddSort : AddSort_MethodGroup
    class AddSort_MethodGroup:
        @typing.overload
        def __call__(self, expr: typing.Any, comparer: IComparer) -> None:...
        @typing.overload
        def __call__(self, expr: typing.Any, order: XmlSortOrder, caseOrder: XmlCaseOrder, lang: str, dataType: XmlDataType) -> None:...

    # Skipped Compile due to it being static, abstract and generic.

    Compile : Compile_MethodGroup
    class Compile_MethodGroup:
        @typing.overload
        def __call__(self, xpath: str) -> XPathExpression:...
        @typing.overload
        def __call__(self, xpath: str, nsResolver: IXmlNamespaceResolver) -> XPathExpression:...

    # Skipped SetContext due to it being static, abstract and generic.

    SetContext : SetContext_MethodGroup
    class SetContext_MethodGroup:
        @typing.overload
        def __call__(self, nsManager: XmlNamespaceManager) -> None:...
        @typing.overload
        def __call__(self, nsResolver: IXmlNamespaceResolver) -> None:...



class XPathItem(abc.ABC):
    @property
    def IsNode(self) -> bool: ...
    @property
    def TypedValue(self) -> typing.Any: ...
    @property
    def Value(self) -> str: ...
    @property
    def ValueAsBoolean(self) -> bool: ...
    @property
    def ValueAsDateTime(self) -> DateTime: ...
    @property
    def ValueAsDouble(self) -> float: ...
    @property
    def ValueAsInt(self) -> int: ...
    @property
    def ValueAsLong(self) -> int: ...
    @property
    def ValueType(self) -> typing.Type[typing.Any]: ...
    @property
    def XmlType(self) -> XmlSchemaType: ...
    # Skipped ValueAs due to it being static, abstract and generic.

    ValueAs : ValueAs_MethodGroup
    class ValueAs_MethodGroup:
        @typing.overload
        def __call__(self, returnType: typing.Type[typing.Any]) -> typing.Any:...
        @typing.overload
        def __call__(self, returnType: typing.Type[typing.Any], nsResolver: IXmlNamespaceResolver) -> typing.Any:...



class XPathNamespaceScope(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    All : XPathNamespaceScope # 0
    ExcludeXml : XPathNamespaceScope # 1
    Local : XPathNamespaceScope # 2


class XPathNavigator(XPathItem, IXmlNamespaceResolver, IXPathNavigable, ICloneable):
    @property
    def BaseURI(self) -> str: ...
    @property
    def CanEdit(self) -> bool: ...
    @property
    def HasAttributes(self) -> bool: ...
    @property
    def HasChildren(self) -> bool: ...
    @property
    def InnerXml(self) -> str: ...
    @InnerXml.setter
    def InnerXml(self, value: str) -> str: ...
    @property
    def IsEmptyElement(self) -> bool: ...
    @property
    def IsNode(self) -> bool: ...
    @property
    def LocalName(self) -> str: ...
    @property
    def Name(self) -> str: ...
    @property
    def NamespaceURI(self) -> str: ...
    @property
    def NameTable(self) -> XmlNameTable: ...
    @classmethod
    @property
    def NavigatorComparer(cls) -> IEqualityComparer: ...
    @property
    def NodeType(self) -> XPathNodeType: ...
    @property
    def OuterXml(self) -> str: ...
    @OuterXml.setter
    def OuterXml(self, value: str) -> str: ...
    @property
    def Prefix(self) -> str: ...
    @property
    def SchemaInfo(self) -> IXmlSchemaInfo: ...
    @property
    def TypedValue(self) -> typing.Any: ...
    @property
    def UnderlyingObject(self) -> typing.Any: ...
    @property
    def Value(self) -> str: ...
    @property
    def ValueAsBoolean(self) -> bool: ...
    @property
    def ValueAsDateTime(self) -> DateTime: ...
    @property
    def ValueAsDouble(self) -> float: ...
    @property
    def ValueAsInt(self) -> int: ...
    @property
    def ValueAsLong(self) -> int: ...
    @property
    def ValueType(self) -> typing.Type[typing.Any]: ...
    @property
    def XmlLang(self) -> str: ...
    @property
    def XmlType(self) -> XmlSchemaType: ...
    def AppendChildElement(self, prefix: str, localName: str, namespaceURI: str, value: str) -> None: ...
    def CheckValidity(self, schemas: XmlSchemaSet, validationEventHandler: ValidationEventHandler) -> bool: ...
    @abc.abstractmethod
    def Clone(self) -> XPathNavigator: ...
    def ComparePosition(self, nav: XPathNavigator) -> XmlNodeOrder: ...
    def Compile(self, xpath: str) -> XPathExpression: ...
    def CreateAttribute(self, prefix: str, localName: str, namespaceURI: str, value: str) -> None: ...
    def CreateAttributes(self) -> XmlWriter: ...
    def CreateNavigator(self) -> XPathNavigator: ...
    def DeleteRange(self, lastSiblingToDelete: XPathNavigator) -> None: ...
    def DeleteSelf(self) -> None: ...
    def GetAttribute(self, localName: str, namespaceURI: str) -> str: ...
    def GetNamespace(self, name: str) -> str: ...
    def GetNamespacesInScope(self, scope: XmlNamespaceScope) -> IDictionary_2[str, str]: ...
    def InsertElementAfter(self, prefix: str, localName: str, namespaceURI: str, value: str) -> None: ...
    def InsertElementBefore(self, prefix: str, localName: str, namespaceURI: str, value: str) -> None: ...
    def IsDescendant(self, nav: XPathNavigator) -> bool: ...
    @abc.abstractmethod
    def IsSamePosition(self, other: XPathNavigator) -> bool: ...
    def LookupNamespace(self, prefix: str) -> str: ...
    def LookupPrefix(self, namespaceURI: str) -> str: ...
    @abc.abstractmethod
    def MoveTo(self, other: XPathNavigator) -> bool: ...
    def MoveToAttribute(self, localName: str, namespaceURI: str) -> bool: ...
    def MoveToFirst(self) -> bool: ...
    @abc.abstractmethod
    def MoveToFirstAttribute(self) -> bool: ...
    @abc.abstractmethod
    def MoveToFirstChild(self) -> bool: ...
    @abc.abstractmethod
    def MoveToId(self, id: str) -> bool: ...
    def MoveToNamespace(self, name: str) -> bool: ...
    @abc.abstractmethod
    def MoveToNextAttribute(self) -> bool: ...
    @abc.abstractmethod
    def MoveToParent(self) -> bool: ...
    @abc.abstractmethod
    def MoveToPrevious(self) -> bool: ...
    def MoveToRoot(self) -> None: ...
    def PrependChildElement(self, prefix: str, localName: str, namespaceURI: str, value: str) -> None: ...
    def ReadSubtree(self) -> XmlReader: ...
    def ReplaceRange(self, lastSiblingToReplace: XPathNavigator) -> XmlWriter: ...
    def SetTypedValue(self, typedValue: typing.Any) -> None: ...
    def SetValue(self, value: str) -> None: ...
    def ToString(self) -> str: ...
    def ValueAs(self, returnType: typing.Type[typing.Any], nsResolver: IXmlNamespaceResolver) -> typing.Any: ...
    def WriteSubtree(self, writer: XmlWriter) -> None: ...
    # Skipped AppendChild due to it being static, abstract and generic.

    AppendChild : AppendChild_MethodGroup
    class AppendChild_MethodGroup:
        @typing.overload
        def __call__(self) -> XmlWriter:...
        @typing.overload
        def __call__(self, newChild: str) -> None:...
        @typing.overload
        def __call__(self, newChild: XPathNavigator) -> None:...
        @typing.overload
        def __call__(self, newChild: XmlReader) -> None:...

    # Skipped Evaluate due to it being static, abstract and generic.

    Evaluate : Evaluate_MethodGroup
    class Evaluate_MethodGroup:
        @typing.overload
        def __call__(self, xpath: str) -> typing.Any:...
        @typing.overload
        def __call__(self, expr: XPathExpression) -> typing.Any:...
        @typing.overload
        def __call__(self, xpath: str, resolver: IXmlNamespaceResolver) -> typing.Any:...
        @typing.overload
        def __call__(self, expr: XPathExpression, context: XPathNodeIterator) -> typing.Any:...

    # Skipped InsertAfter due to it being static, abstract and generic.

    InsertAfter : InsertAfter_MethodGroup
    class InsertAfter_MethodGroup:
        @typing.overload
        def __call__(self) -> XmlWriter:...
        @typing.overload
        def __call__(self, newSibling: str) -> None:...
        @typing.overload
        def __call__(self, newSibling: XPathNavigator) -> None:...
        @typing.overload
        def __call__(self, newSibling: XmlReader) -> None:...

    # Skipped InsertBefore due to it being static, abstract and generic.

    InsertBefore : InsertBefore_MethodGroup
    class InsertBefore_MethodGroup:
        @typing.overload
        def __call__(self) -> XmlWriter:...
        @typing.overload
        def __call__(self, newSibling: str) -> None:...
        @typing.overload
        def __call__(self, newSibling: XPathNavigator) -> None:...
        @typing.overload
        def __call__(self, newSibling: XmlReader) -> None:...

    # Skipped Matches due to it being static, abstract and generic.

    Matches : Matches_MethodGroup
    class Matches_MethodGroup:
        @typing.overload
        def __call__(self, xpath: str) -> bool:...
        @typing.overload
        def __call__(self, expr: XPathExpression) -> bool:...

    # Skipped MoveToChild due to it being static, abstract and generic.

    MoveToChild : MoveToChild_MethodGroup
    class MoveToChild_MethodGroup:
        @typing.overload
        def __call__(self, type: XPathNodeType) -> bool:...
        @typing.overload
        def __call__(self, localName: str, namespaceURI: str) -> bool:...

    # Skipped MoveToFirstNamespace due to it being static, abstract and generic.

    MoveToFirstNamespace : MoveToFirstNamespace_MethodGroup
    class MoveToFirstNamespace_MethodGroup:
        @typing.overload
        def __call__(self) -> bool:...
        @typing.overload
        def __call__(self, namespaceScope: XPathNamespaceScope) -> bool:...

    # Skipped MoveToFollowing due to it being static, abstract and generic.

    MoveToFollowing : MoveToFollowing_MethodGroup
    class MoveToFollowing_MethodGroup:
        @typing.overload
        def __call__(self, type: XPathNodeType) -> bool:...
        @typing.overload
        def __call__(self, type: XPathNodeType, end: XPathNavigator) -> bool:...
        @typing.overload
        def __call__(self, localName: str, namespaceURI: str) -> bool:...
        @typing.overload
        def __call__(self, localName: str, namespaceURI: str, end: XPathNavigator) -> bool:...

    # Skipped MoveToNext due to it being static, abstract and generic.

    MoveToNext : MoveToNext_MethodGroup
    class MoveToNext_MethodGroup:
        @typing.overload
        def __call__(self) -> bool:...
        @typing.overload
        def __call__(self, type: XPathNodeType) -> bool:...
        @typing.overload
        def __call__(self, localName: str, namespaceURI: str) -> bool:...

    # Skipped MoveToNextNamespace due to it being static, abstract and generic.

    MoveToNextNamespace : MoveToNextNamespace_MethodGroup
    class MoveToNextNamespace_MethodGroup:
        @typing.overload
        def __call__(self) -> bool:...
        @typing.overload
        def __call__(self, namespaceScope: XPathNamespaceScope) -> bool:...

    # Skipped PrependChild due to it being static, abstract and generic.

    PrependChild : PrependChild_MethodGroup
    class PrependChild_MethodGroup:
        @typing.overload
        def __call__(self) -> XmlWriter:...
        @typing.overload
        def __call__(self, newChild: str) -> None:...
        @typing.overload
        def __call__(self, newChild: XPathNavigator) -> None:...
        @typing.overload
        def __call__(self, newChild: XmlReader) -> None:...

    # Skipped ReplaceSelf due to it being static, abstract and generic.

    ReplaceSelf : ReplaceSelf_MethodGroup
    class ReplaceSelf_MethodGroup:
        @typing.overload
        def __call__(self, newNode: str) -> None:...
        @typing.overload
        def __call__(self, newNode: XPathNavigator) -> None:...
        @typing.overload
        def __call__(self, newNode: XmlReader) -> None:...

    # Skipped Select due to it being static, abstract and generic.

    Select : Select_MethodGroup
    class Select_MethodGroup:
        @typing.overload
        def __call__(self, xpath: str) -> XPathNodeIterator:...
        @typing.overload
        def __call__(self, expr: XPathExpression) -> XPathNodeIterator:...
        @typing.overload
        def __call__(self, xpath: str, resolver: IXmlNamespaceResolver) -> XPathNodeIterator:...

    # Skipped SelectAncestors due to it being static, abstract and generic.

    SelectAncestors : SelectAncestors_MethodGroup
    class SelectAncestors_MethodGroup:
        @typing.overload
        def __call__(self, type: XPathNodeType, matchSelf: bool) -> XPathNodeIterator:...
        @typing.overload
        def __call__(self, name: str, namespaceURI: str, matchSelf: bool) -> XPathNodeIterator:...

    # Skipped SelectChildren due to it being static, abstract and generic.

    SelectChildren : SelectChildren_MethodGroup
    class SelectChildren_MethodGroup:
        @typing.overload
        def __call__(self, type: XPathNodeType) -> XPathNodeIterator:...
        @typing.overload
        def __call__(self, name: str, namespaceURI: str) -> XPathNodeIterator:...

    # Skipped SelectDescendants due to it being static, abstract and generic.

    SelectDescendants : SelectDescendants_MethodGroup
    class SelectDescendants_MethodGroup:
        @typing.overload
        def __call__(self, type: XPathNodeType, matchSelf: bool) -> XPathNodeIterator:...
        @typing.overload
        def __call__(self, name: str, namespaceURI: str, matchSelf: bool) -> XPathNodeIterator:...

    # Skipped SelectSingleNode due to it being static, abstract and generic.

    SelectSingleNode : SelectSingleNode_MethodGroup
    class SelectSingleNode_MethodGroup:
        @typing.overload
        def __call__(self, xpath: str) -> XPathNavigator:...
        @typing.overload
        def __call__(self, expression: XPathExpression) -> XPathNavigator:...
        @typing.overload
        def __call__(self, xpath: str, resolver: IXmlNamespaceResolver) -> XPathNavigator:...



class XPathNodeIterator(IEnumerable, ICloneable, abc.ABC):
    @property
    def Count(self) -> int: ...
    @property
    def Current(self) -> XPathNavigator: ...
    @property
    def CurrentPosition(self) -> int: ...
    @abc.abstractmethod
    def Clone(self) -> XPathNodeIterator: ...
    def GetEnumerator(self) -> IEnumerator: ...
    @abc.abstractmethod
    def MoveNext(self) -> bool: ...


class XPathNodeType(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Root : XPathNodeType # 0
    Element : XPathNodeType # 1
    Attribute : XPathNodeType # 2
    Namespace : XPathNodeType # 3
    Text : XPathNodeType # 4
    SignificantWhitespace : XPathNodeType # 5
    Whitespace : XPathNodeType # 6
    ProcessingInstruction : XPathNodeType # 7
    Comment : XPathNodeType # 8
    All : XPathNodeType # 9


class XPathResultType(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Number : XPathResultType # 0
    String : XPathResultType # 1
    Navigator : XPathResultType # 1
    Boolean : XPathResultType # 2
    NodeSet : XPathResultType # 3
    Any : XPathResultType # 5
    Error : XPathResultType # 6




import io
from functools import cache
from ruamel.yaml import YAML, yaml_object, add_representer
from . import yaml

from typing import TYPE_CHECKING, Optional, Union, Literal, Any, Dict

if TYPE_CHECKING:
    from .data_model import DataModel

from SIMULTAN.Data.Taxonomy import SimTaxonomyEntryReference, SimTaxonomyEntry, SimTaxonomyEntryReference, SimTaxonomy


@yaml_object(yaml)
class Content(object):

    yaml_tag = u'!Content'

    def __init__(self,
                 text_or_key: str,
                 name: Optional[str] = None,
                 property_name: Optional[str] = None,
                 slot_extension: Optional[str] = None,
                 type: Optional[Any] = None,
                 unit: Optional[str] = None,
                 documentation: Optional[str] = None,
                 component_policy: Optional[Literal['reference', 'subcomponent']] = 'subcomponent',
                 *args,
                 **kwargs):
        """
        Define a content/parameter/property of a taxonomy entry in the taxonomy map.

        :param args:
        :Keyword Arguments
            * *name* (``str``) -- name of the Taxonomy Entry in SIMULTAN
            * *text_or_key* (``str``) -- text or key of the content/parameter/property
            * *property_name* (``str``) -- name of the generated property
            * *slot_extension* (``str``) -- slot extension of the content/parameter/property
            * *type* (``str``) -- type of the content/parameter/property
            * *unit* (``str``) -- unit of the content/parameter/property
            * *documentation* (``str``) -- documentation of the content/parameter/property
            * *component_policy* (``str``) -- component add policy of the content/parameter/property, 'reference' or 'subcomponent'
        """

        self.name: str = name if name is not None else text_or_key # name of the Taxonomy Entry in SIMULTAN
        self.text_or_key: str = text_or_key          # text or key of the content/parameter/property
        self.property_name: str = property_name if property_name is not None else text_or_key  # name of the generated property in the python class
        self.slot_extension: str = slot_extension    # slot extension of the content/parameter/property
        self.type: Optional[Any] = type                        # type of the content/parameter/property
        self.unit: Optional[str] = unit                       # unit of the content/parameter/property
        self.documentation: Optional[str] = documentation       # documentation of the content/parameter/property
        self.component_policy: Literal['reference', 'subcomponent'] = component_policy  # component add policy of the content/parameter/property, 'reference' or 'subcomponent'

        self._taxonomies = {}
        self._taxonomy_entries = {}
        self._taxonomy_map: Optional['TaxonomyMap'] = kwargs.get('taxonomy_map', None)

        self.taxonomy_key: Optional[SimTaxonomy] = kwargs.get('taxonomy_key')

        add_kwargs = kwargs.copy()
        _ = [add_kwargs.pop(key, None) for key in ['taxonomy_name', 'taxonomy_key', 'taxonomy_entry_name',
                                                   'taxonomy_entry_key', 'content', 'documentation', 'python_mapper',
                                                   'mapped_class', 'unit', 'type', 'component_policy',
                                                   'text_or_key', 'property_name', 'slot_extension']]

        self.additional_attributes: dict[Any, Any] = add_kwargs

        self.taxonomy_key: Optional[SimTaxonomy]  = kwargs.get('taxonomy_key')
        self.taxonomy_name: str = kwargs.get('taxonomy_name')


    def get_taxonomie_entry(self, data_model: 'DataModel'):
        if self._taxonomy_entries.get(data_model, None) is None:

            if self.taxonomy_key is None:
                taxonomy = self._taxonomy_map.get_or_create_simultan_taxonomy(data_model=data_model, create=True)
            else:
                taxonomy = data_model.get_or_create_taxonomy(taxonomy_name=self.taxonomy_name,
                                                             taxonomy_key=self.taxonomy_key,
                                                             create=True)

            self._taxonomy_entries[data_model] = data_model.get_or_create_taxonomy_entry(name=self.name,
                                                                                         key=self.text_or_key,
                                                                                         description=self.documentation,
                                                                                         sim_taxonomy=taxonomy)
        return self._taxonomy_entries[data_model]

    def __repr__(self):
        return (f"Content({self.name}, {self.text_or_key}, "
                f"{self.property_name}, {self.slot_extension}, "
                f"{self.type}, {self.unit}, "
                f"{self.documentation}, {self.component_policy})")


verbose_output = False


@yaml_object(yaml)
class TaxonomyMap(object):

    yaml_tag = u'!TaxonomyMap'

    @classmethod
    def from_yaml_file(cls, filename):
        with open(filename, mode='r') as f:
            taxonomie_map = yaml.load(f)
        return taxonomie_map[0]

    def __init__(self,
                 taxonomy_key: str,
                 taxonomy_entry_key: str,
                 taxonomy_name: Optional[str] = None,
                 taxonomy_entry_name: Optional[str] = None,
                 content: Optional[list[Content]] = None,
                 documentation: Optional[str] = None,
                 *args,
                 **kwargs):

        self._content = []
        self._content_dict = {}
        self._parameter_taxonomy_entry_dict = {}

        self.taxonomy_key = taxonomy_key
        self.taxonomy_name = taxonomy_name if taxonomy_name is not None else taxonomy_key


        if self.taxonomy_key == taxonomy_entry_key:
            raise ValueError('taxonomy_key and taxonomy_entry_key must be different')

        self.taxonomy_entry_key = taxonomy_entry_key
        self.taxonomy_entry_name = taxonomy_entry_name if taxonomy_entry_name is not None else taxonomy_entry_key

        self.content = content if content is not None else []
        self.documentation = documentation

        self.python_mapper = kwargs.get('python_mapper', None)
        self.mapped_class = kwargs.get('mapped_class', None)

        self._taxonomies: Dict[DataModel, SimTaxonomy] = {}
        self._taxonomy_entries: Dict[DataModel, SimTaxonomyEntry] = {}

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value: list[Content]):
        self._content = value
        for content in self._content:
            content._taxonomy_map = self
        self._content_dict = {}

        self.get_content_by_property_name.cache_clear()
        self.get_content_by_text_or_key.cache_clear()

    @property
    def content_dict(self):
        if self._content_dict == {}:
            for content in self._content:
                self._content_dict[content.text_or_key] = content
        return self._content_dict

    @property
    def parameter_taxonomy_entry_dict(self):
        if not self._parameter_taxonomy_entry_dict:
            self._parameter_taxonomy_entry_dict = {content.property_name: content.text_or_key for content in self._content}
        return self._parameter_taxonomy_entry_dict

    def write(self, filename=None):
        if filename is not None:
            with open(filename, mode='w') as f:
                yaml.dump([self], f)
        else:
            f = io.StringIO()
            yaml.dump([self], f)
            return f.getvalue()

    def __getstate__(self):
        data = self.__dict__.copy()
        del data['python_mapper']
        del data['mapped_class']
        return data

    def add_content(self, content: Content):
        self._content.append(content)
        content._taxonomy_map = self
        self._content_dict = {}

    def get_or_create_simultan_taxonomy_entry(self,
                                              data_model: 'DataModel',
                                              create=True):

        if self._taxonomy_entries.get(data_model, None) is None:

            taxonomy_entries = data_model.get_taxonomy_entries(taxonomy=self.taxonomy_key)
            if self.taxonomy_entry_key in taxonomy_entries.keys():
                self._taxonomy_entries[data_model] = taxonomy_entries[self.taxonomy_entry_key]
            elif create:
                self._taxonomy_entries[data_model] = data_model.get_or_create_taxonomy_entry(
                    key=self.taxonomy_entry_key,
                    name=self.taxonomy_entry_name,
                    sim_taxonomy=self.get_or_create_simultan_taxonomy(data_model=data_model,
                                                                      create=create))
        return self._taxonomy_entries[data_model]

    def get_or_create_simultan_taxonomy(self,
                                        data_model: 'DataModel',
                                        create=True) -> SimTaxonomy:
        if self._taxonomies.get(data_model, None) is None:
            self._taxonomies[data_model] = data_model.get_or_create_taxonomy(
                taxonomy_name=self.taxonomy_name,
                taxonomy_key=self.taxonomy_key,
                description=self.documentation,
                create=create)

        return self._taxonomies[data_model]

    def get_slot(self, data_model: 'DataModel'):
        return SimTaxonomyEntryReference(self.get_or_create_simultan_taxonomy_entry(data_model=data_model,
                                                                                    create=True))

    @cache
    def get_content_by_property_name(self, property_name: str):
        return next((x for x in self.content if x.property_name == property_name), None)

    @cache
    def get_content_by_text_or_key(self, text_or_key: str):
        return next((x for x in self.content if x.text_or_key == text_or_key), None)



import typing, clr, abc
from System.Collections.ObjectModel import ObservableCollection_1
from System.Collections.Generic import IEnumerable_1, IReadOnlyCollection_1, IReadOnlyDictionary_2, KeyValuePair_2, ICollection_1, IEnumerator_1, IDictionary_2
from System.Collections.Specialized import INotifyCollectionChanged
from System.ComponentModel import INotifyPropertyChanged
from System import Comparison_1

class ElectivelyObservableCollection_GenericClasses(abc.ABCMeta):
    Generic_ElectivelyObservableCollection_GenericClasses_ElectivelyObservableCollection_1_T = typing.TypeVar('Generic_ElectivelyObservableCollection_GenericClasses_ElectivelyObservableCollection_1_T')
    def __getitem__(self, types : typing.Type[Generic_ElectivelyObservableCollection_GenericClasses_ElectivelyObservableCollection_1_T]) -> typing.Type[ElectivelyObservableCollection_1[Generic_ElectivelyObservableCollection_GenericClasses_ElectivelyObservableCollection_1_T]]: ...

ElectivelyObservableCollection : ElectivelyObservableCollection_GenericClasses

ElectivelyObservableCollection_1_T = typing.TypeVar('ElectivelyObservableCollection_1_T')
class ElectivelyObservableCollection_1(typing.Generic[ElectivelyObservableCollection_1_T], ObservableCollection_1[ElectivelyObservableCollection_1_T], IReadOnlyObservableCollection_1[ElectivelyObservableCollection_1_T]):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, collection: IEnumerable_1[ElectivelyObservableCollection_1_T]) -> None: ...
    @property
    def Count(self) -> int: ...
    @property
    def Item(self) -> ElectivelyObservableCollection_1_T: ...
    @Item.setter
    def Item(self, value: ElectivelyObservableCollection_1_T) -> ElectivelyObservableCollection_1_T: ...
    @property
    def SuppressNotification(self) -> bool: ...
    @SuppressNotification.setter
    def SuppressNotification(self, value: bool) -> bool: ...


class IReadOnlyObservableCollection_GenericClasses(abc.ABCMeta):
    Generic_IReadOnlyObservableCollection_GenericClasses_IReadOnlyObservableCollection_1_T = typing.TypeVar('Generic_IReadOnlyObservableCollection_GenericClasses_IReadOnlyObservableCollection_1_T')
    def __getitem__(self, types : typing.Type[Generic_IReadOnlyObservableCollection_GenericClasses_IReadOnlyObservableCollection_1_T]) -> typing.Type[IReadOnlyObservableCollection_1[Generic_IReadOnlyObservableCollection_GenericClasses_IReadOnlyObservableCollection_1_T]]: ...

IReadOnlyObservableCollection : IReadOnlyObservableCollection_GenericClasses

IReadOnlyObservableCollection_1_T = typing.TypeVar('IReadOnlyObservableCollection_1_T')
class IReadOnlyObservableCollection_1(typing.Generic[IReadOnlyObservableCollection_1_T], IReadOnlyCollection_1[IReadOnlyObservableCollection_1_T], INotifyCollectionChanged, typing.Protocol):
    pass


class IReadonlyObservableDictionary_GenericClasses(abc.ABCMeta):
    Generic_IReadonlyObservableDictionary_GenericClasses_IReadonlyObservableDictionary_2_TKey = typing.TypeVar('Generic_IReadonlyObservableDictionary_GenericClasses_IReadonlyObservableDictionary_2_TKey')
    Generic_IReadonlyObservableDictionary_GenericClasses_IReadonlyObservableDictionary_2_TValue = typing.TypeVar('Generic_IReadonlyObservableDictionary_GenericClasses_IReadonlyObservableDictionary_2_TValue')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_IReadonlyObservableDictionary_GenericClasses_IReadonlyObservableDictionary_2_TKey], typing.Type[Generic_IReadonlyObservableDictionary_GenericClasses_IReadonlyObservableDictionary_2_TValue]]) -> typing.Type[IReadonlyObservableDictionary_2[Generic_IReadonlyObservableDictionary_GenericClasses_IReadonlyObservableDictionary_2_TKey, Generic_IReadonlyObservableDictionary_GenericClasses_IReadonlyObservableDictionary_2_TValue]]: ...

IReadonlyObservableDictionary : IReadonlyObservableDictionary_GenericClasses

IReadonlyObservableDictionary_2_TKey = typing.TypeVar('IReadonlyObservableDictionary_2_TKey')
IReadonlyObservableDictionary_2_TValue = typing.TypeVar('IReadonlyObservableDictionary_2_TValue')
class IReadonlyObservableDictionary_2(typing.Generic[IReadonlyObservableDictionary_2_TKey, IReadonlyObservableDictionary_2_TValue], IReadOnlyDictionary_2[IReadonlyObservableDictionary_2_TKey, IReadonlyObservableDictionary_2_TValue], INotifyCollectionChanged, typing.Protocol):
    pass


class MultiDictionary_GenericClasses(abc.ABCMeta):
    Generic_MultiDictionary_GenericClasses_MultiDictionary_2_TKey = typing.TypeVar('Generic_MultiDictionary_GenericClasses_MultiDictionary_2_TKey')
    Generic_MultiDictionary_GenericClasses_MultiDictionary_2_TValue = typing.TypeVar('Generic_MultiDictionary_GenericClasses_MultiDictionary_2_TValue')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_MultiDictionary_GenericClasses_MultiDictionary_2_TKey], typing.Type[Generic_MultiDictionary_GenericClasses_MultiDictionary_2_TValue]]) -> typing.Type[MultiDictionary_2[Generic_MultiDictionary_GenericClasses_MultiDictionary_2_TKey, Generic_MultiDictionary_GenericClasses_MultiDictionary_2_TValue]]: ...

MultiDictionary : MultiDictionary_GenericClasses

MultiDictionary_2_TKey = typing.TypeVar('MultiDictionary_2_TKey')
MultiDictionary_2_TValue = typing.TypeVar('MultiDictionary_2_TValue')
class MultiDictionary_2(typing.Generic[MultiDictionary_2_TKey, MultiDictionary_2_TValue], IEnumerable_1[KeyValuePair_2[MultiDictionary_2_TKey, IEnumerable_1[MultiDictionary_2_TValue]]]):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, original: MultiDictionary_2[MultiDictionary_2_TKey, MultiDictionary_2_TValue]) -> None: ...
    @property
    def Item(self) -> IEnumerable_1[MultiDictionary_2_TValue]: ...
    @property
    def Keys(self) -> ICollection_1[MultiDictionary_2_TKey]: ...
    def Clear(self) -> None: ...
    def ContainsKey(self, key: MultiDictionary_2_TKey) -> bool: ...
    def GetEnumerator(self) -> IEnumerator_1[KeyValuePair_2[MultiDictionary_2_TKey, IEnumerable_1[MultiDictionary_2_TValue]]]: ...
    def TryGetValues(self, key: MultiDictionary_2_TKey, values: clr.Reference[IEnumerable_1[MultiDictionary_2_TValue]]) -> bool: ...
    # Skipped Add due to it being static, abstract and generic.

    Add : Add_MethodGroup[MultiDictionary_2_TKey, MultiDictionary_2_TValue]
    Add_MethodGroup_MultiDictionary_2_TKey = typing.TypeVar('Add_MethodGroup_MultiDictionary_2_TKey')
    Add_MethodGroup_MultiDictionary_2_TValue = typing.TypeVar('Add_MethodGroup_MultiDictionary_2_TValue')
    class Add_MethodGroup(typing.Generic[Add_MethodGroup_MultiDictionary_2_TKey, Add_MethodGroup_MultiDictionary_2_TValue]):
        Add_MethodGroup_MultiDictionary_2_TKey = MultiDictionary_2.Add_MethodGroup_MultiDictionary_2_TKey
        Add_MethodGroup_MultiDictionary_2_TValue = MultiDictionary_2.Add_MethodGroup_MultiDictionary_2_TValue
        @typing.overload
        def __call__(self, key: Add_MethodGroup_MultiDictionary_2_TKey, values: IEnumerable_1[Add_MethodGroup_MultiDictionary_2_TValue]) -> None:...
        @typing.overload
        def __call__(self, key: Add_MethodGroup_MultiDictionary_2_TKey, value: Add_MethodGroup_MultiDictionary_2_TValue) -> None:...

    # Skipped Remove due to it being static, abstract and generic.

    Remove : Remove_MethodGroup[MultiDictionary_2_TKey, MultiDictionary_2_TValue]
    Remove_MethodGroup_MultiDictionary_2_TKey = typing.TypeVar('Remove_MethodGroup_MultiDictionary_2_TKey')
    Remove_MethodGroup_MultiDictionary_2_TValue = typing.TypeVar('Remove_MethodGroup_MultiDictionary_2_TValue')
    class Remove_MethodGroup(typing.Generic[Remove_MethodGroup_MultiDictionary_2_TKey, Remove_MethodGroup_MultiDictionary_2_TValue]):
        Remove_MethodGroup_MultiDictionary_2_TKey = MultiDictionary_2.Remove_MethodGroup_MultiDictionary_2_TKey
        Remove_MethodGroup_MultiDictionary_2_TValue = MultiDictionary_2.Remove_MethodGroup_MultiDictionary_2_TValue
        @typing.overload
        def __call__(self, key: Remove_MethodGroup_MultiDictionary_2_TKey) -> bool:...
        @typing.overload
        def __call__(self, key: Remove_MethodGroup_MultiDictionary_2_TKey, value: Remove_MethodGroup_MultiDictionary_2_TValue) -> bool:...



class MultiDictionaryOptimized_GenericClasses(abc.ABCMeta):
    Generic_MultiDictionaryOptimized_GenericClasses_MultiDictionaryOptimized_2_TKey = typing.TypeVar('Generic_MultiDictionaryOptimized_GenericClasses_MultiDictionaryOptimized_2_TKey')
    Generic_MultiDictionaryOptimized_GenericClasses_MultiDictionaryOptimized_2_TValue = typing.TypeVar('Generic_MultiDictionaryOptimized_GenericClasses_MultiDictionaryOptimized_2_TValue')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_MultiDictionaryOptimized_GenericClasses_MultiDictionaryOptimized_2_TKey], typing.Type[Generic_MultiDictionaryOptimized_GenericClasses_MultiDictionaryOptimized_2_TValue]]) -> typing.Type[MultiDictionaryOptimized_2[Generic_MultiDictionaryOptimized_GenericClasses_MultiDictionaryOptimized_2_TKey, Generic_MultiDictionaryOptimized_GenericClasses_MultiDictionaryOptimized_2_TValue]]: ...

MultiDictionaryOptimized : MultiDictionaryOptimized_GenericClasses

MultiDictionaryOptimized_2_TKey = typing.TypeVar('MultiDictionaryOptimized_2_TKey')
MultiDictionaryOptimized_2_TValue = typing.TypeVar('MultiDictionaryOptimized_2_TValue')
class MultiDictionaryOptimized_2(typing.Generic[MultiDictionaryOptimized_2_TKey, MultiDictionaryOptimized_2_TValue]):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, original: MultiDictionaryOptimized_2[MultiDictionaryOptimized_2_TKey, MultiDictionaryOptimized_2_TValue]) -> None: ...
    @property
    def Item(self) -> IReadOnlyCollection_1[MultiDictionaryOptimized_2_TValue]: ...
    def Add(self, key: MultiDictionaryOptimized_2_TKey, value: MultiDictionaryOptimized_2_TValue) -> None: ...
    def Clear(self) -> None: ...
    def ContainsKey(self, key: MultiDictionaryOptimized_2_TKey) -> bool: ...
    def TryGetValues(self, key: MultiDictionaryOptimized_2_TKey, values: clr.Reference[IEnumerable_1[MultiDictionaryOptimized_2_TValue]]) -> bool: ...
    # Skipped Remove due to it being static, abstract and generic.

    Remove : Remove_MethodGroup[MultiDictionaryOptimized_2_TKey, MultiDictionaryOptimized_2_TValue]
    Remove_MethodGroup_MultiDictionaryOptimized_2_TKey = typing.TypeVar('Remove_MethodGroup_MultiDictionaryOptimized_2_TKey')
    Remove_MethodGroup_MultiDictionaryOptimized_2_TValue = typing.TypeVar('Remove_MethodGroup_MultiDictionaryOptimized_2_TValue')
    class Remove_MethodGroup(typing.Generic[Remove_MethodGroup_MultiDictionaryOptimized_2_TKey, Remove_MethodGroup_MultiDictionaryOptimized_2_TValue]):
        Remove_MethodGroup_MultiDictionaryOptimized_2_TKey = MultiDictionaryOptimized_2.Remove_MethodGroup_MultiDictionaryOptimized_2_TKey
        Remove_MethodGroup_MultiDictionaryOptimized_2_TValue = MultiDictionaryOptimized_2.Remove_MethodGroup_MultiDictionaryOptimized_2_TValue
        @typing.overload
        def __call__(self, key: Remove_MethodGroup_MultiDictionaryOptimized_2_TKey) -> bool:...
        @typing.overload
        def __call__(self, key: Remove_MethodGroup_MultiDictionaryOptimized_2_TKey, value: Remove_MethodGroup_MultiDictionaryOptimized_2_TValue) -> bool:...



class ObservableDictionary_GenericClasses(abc.ABCMeta):
    Generic_ObservableDictionary_GenericClasses_ObservableDictionary_2_TKey = typing.TypeVar('Generic_ObservableDictionary_GenericClasses_ObservableDictionary_2_TKey')
    Generic_ObservableDictionary_GenericClasses_ObservableDictionary_2_TValue = typing.TypeVar('Generic_ObservableDictionary_GenericClasses_ObservableDictionary_2_TValue')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_ObservableDictionary_GenericClasses_ObservableDictionary_2_TKey], typing.Type[Generic_ObservableDictionary_GenericClasses_ObservableDictionary_2_TValue]]) -> typing.Type[ObservableDictionary_2[Generic_ObservableDictionary_GenericClasses_ObservableDictionary_2_TKey, Generic_ObservableDictionary_GenericClasses_ObservableDictionary_2_TValue]]: ...

ObservableDictionary : ObservableDictionary_GenericClasses

ObservableDictionary_2_TKey = typing.TypeVar('ObservableDictionary_2_TKey')
ObservableDictionary_2_TValue = typing.TypeVar('ObservableDictionary_2_TValue')
class ObservableDictionary_2(typing.Generic[ObservableDictionary_2_TKey, ObservableDictionary_2_TValue], IReadonlyObservableDictionary_2[ObservableDictionary_2_TKey, ObservableDictionary_2_TValue], IDictionary_2[ObservableDictionary_2_TKey, ObservableDictionary_2_TValue], INotifyPropertyChanged):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, dictionary: IDictionary_2[ObservableDictionary_2_TKey, ObservableDictionary_2_TValue]) -> None: ...
    @typing.overload
    def __init__(self, input: IEnumerable_1[KeyValuePair_2[ObservableDictionary_2_TKey, ObservableDictionary_2_TValue]]) -> None: ...
    @typing.overload
    def __init__(self, keys: IEnumerable_1[ObservableDictionary_2_TKey], values: IEnumerable_1[ObservableDictionary_2_TValue]) -> None: ...
    @property
    def Count(self) -> int: ...
    @property
    def Item(self) -> ObservableDictionary_2_TValue: ...
    @Item.setter
    def Item(self, value: ObservableDictionary_2_TValue) -> ObservableDictionary_2_TValue: ...
    @property
    def Keys(self) -> ICollection_1[ObservableDictionary_2_TKey]: ...
    @property
    def Values(self) -> ICollection_1[ObservableDictionary_2_TValue]: ...
    def Add(self, key: ObservableDictionary_2_TKey, value: ObservableDictionary_2_TValue) -> None: ...
    def Clear(self) -> None: ...
    def ContainsKey(self, key: ObservableDictionary_2_TKey) -> bool: ...
    def ContainsValue(self, value: ObservableDictionary_2_TValue) -> bool: ...
    def Remove(self, key: ObservableDictionary_2_TKey) -> bool: ...
    def TryGetValue(self, key: ObservableDictionary_2_TKey, value: clr.Reference[ObservableDictionary_2_TValue]) -> bool: ...
    # Skipped AddRange due to it being static, abstract and generic.

    AddRange : AddRange_MethodGroup[ObservableDictionary_2_TKey, ObservableDictionary_2_TValue]
    AddRange_MethodGroup_ObservableDictionary_2_TKey = typing.TypeVar('AddRange_MethodGroup_ObservableDictionary_2_TKey')
    AddRange_MethodGroup_ObservableDictionary_2_TValue = typing.TypeVar('AddRange_MethodGroup_ObservableDictionary_2_TValue')
    class AddRange_MethodGroup(typing.Generic[AddRange_MethodGroup_ObservableDictionary_2_TKey, AddRange_MethodGroup_ObservableDictionary_2_TValue]):
        AddRange_MethodGroup_ObservableDictionary_2_TKey = ObservableDictionary_2.AddRange_MethodGroup_ObservableDictionary_2_TKey
        AddRange_MethodGroup_ObservableDictionary_2_TValue = ObservableDictionary_2.AddRange_MethodGroup_ObservableDictionary_2_TValue
        @typing.overload
        def __call__(self, range: IEnumerable_1[KeyValuePair_2[AddRange_MethodGroup_ObservableDictionary_2_TKey, AddRange_MethodGroup_ObservableDictionary_2_TValue]]) -> None:...
        @typing.overload
        def __call__(self, keys: IEnumerable_1[AddRange_MethodGroup_ObservableDictionary_2_TKey], values: IEnumerable_1[AddRange_MethodGroup_ObservableDictionary_2_TValue]) -> None:...



class PriorityQueue_GenericClasses(abc.ABCMeta):
    Generic_PriorityQueue_GenericClasses_PriorityQueue_1_T = typing.TypeVar('Generic_PriorityQueue_GenericClasses_PriorityQueue_1_T')
    def __getitem__(self, types : typing.Type[Generic_PriorityQueue_GenericClasses_PriorityQueue_1_T]) -> typing.Type[PriorityQueue_1[Generic_PriorityQueue_GenericClasses_PriorityQueue_1_T]]: ...

PriorityQueue : PriorityQueue_GenericClasses

PriorityQueue_1_T = typing.TypeVar('PriorityQueue_1_T')
class PriorityQueue_1(typing.Generic[PriorityQueue_1_T]):
    # Constructor .ctor(isdesc : Boolean, comparison : Comparison`1) was skipped since it collides with above method
    @typing.overload
    def __init__(self, capacity: int, comparison: Comparison_1[PriorityQueue_1_T]) -> None: ...
    @typing.overload
    def __init__(self, capacity: int, isdesc: bool, comparison: Comparison_1[PriorityQueue_1_T]) -> None: ...
    @typing.overload
    def __init__(self, collection: IEnumerable_1[PriorityQueue_1_T], comparison: Comparison_1[PriorityQueue_1_T]) -> None: ...
    @typing.overload
    def __init__(self, collection: IEnumerable_1[PriorityQueue_1_T], isdesc: bool, comparison: Comparison_1[PriorityQueue_1_T]) -> None: ...
    @typing.overload
    def __init__(self, comparison: Comparison_1[PriorityQueue_1_T]) -> None: ...
    IsDescending : bool
    @property
    def Count(self) -> int: ...
    def Clear(self) -> None: ...
    def Dequeue(self) -> PriorityQueue_1_T: ...
    def Enqueue(self, x: PriorityQueue_1_T) -> None: ...
    def Peek(self) -> PriorityQueue_1_T: ...




from PySimultan2.data_model import DataModel
from PySimultan2.utils import create_taxonomy, get_ot_create_taxonomy_entry

project_path = r'resources/U5.simultan'

data_model = DataModel(project_path=project_path)

taxonomy = create_taxonomy('test_name', 'test_key', data_model=data_model)

taxonomy_entry = get_ot_create_taxonomy_entry('test_taxonomy_entry_name', 'test_taxonomy_entry_key', sim_taxonomy=taxonomy)

data_model.save()
data_model.cleanup()

print('done')
